/*
	Author: wamesbury
	Created: 2020-04-17
	Purpose: Claim Mapping editor & sub-editors
	History:
	2021-01-19 WRA ND-10634 Added PatientStatementLogoPath
	2020-12-08 RMW ND-10622 Added Scenario 9 and Default HIPPS Code fields
	2020-06-10 WRA ND-10216 Added Type of Care field.
*/

define([
	"dojo/_base/declare",
	"thornberry/object/ObjectEditor",
	"thornberry/object/ObjectModel",
	"thornberry/serverRequest",
	"thornberry/object/TableManager",
	"thornberry/object/template",
	"dojo/_base/array",
	"dojo/_base/lang",
	"dojo/aspect",
	"dojo/store/Memory",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/query",
	"dojo/Deferred",
	"dijit/form/NumberTextBox",
	"dijit/form/Button",
	"dijit/Dialog",
	"dijit/form/MultiSelect",
	"gridx/Grid",
	"gridx/core/model/cache/Sync",
	"gridx/modules/select/Row",
	"gridx/modules/VirtualVScroller",
	"gridx/modules/IndirectSelect",
	"gridx/modules/RowHeader",
	"gridx/modules/SingleSort",
	"dijit/_WidgetBase",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin"
], function(declare, ObjectEditor, ObjectModel, serverRequest, TableManager, template, arrayUtils, lang, aspect, Memory,
			domConstruct, domStyle, query, Deferred, NumberTextBox, Button,	Dialog, MultiSelect,
			Grid, Cache, SelectRow, VScroller, IndirectSelect, RowHeader, SingleSort,
			_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin) {

	var revenueCodeRegEx = "\\d{4}";
	var revenueCodeInvalidMessage = "Revenue codes are 4 digits";
	var insuranceTypeOptions = {};
	TableManager.get("FillInGroup~20605").query().forEach(function(type) {
		insuranceTypeOptions[type.id] = type.name;
	});
	if (hospiceAgency != 1) {
		delete insuranceTypeOptions[2];
	}

	var MINDATE = new Date("12/31/1840");

	var claimMappingModel = new ObjectModel({ propertyDefinitions: [
		{ name: "id", type: "text" },
		{ name: "Name", type: "text", maxLength: 50, required: true },
		{ name: "Status", type: "enum", displayIndex: { "active": "Active", "inactive": "Inactive" }, required: true },
		{ name: "InsuranceType", type: "enum", displayIndex: insuranceTypeOptions, required: true },
		{ name: "ClaimForm", type: "tableItem", tableName: "Tables.Insurance.ClaimForm", required: true },
		{ name: "PrintWithFormImage", type: "boolean" },
		
		// Generic electronic options
		{ name: "InterchangeSenderID", type: "text", required: true, maxLength: 15 },
		{ name: "InterchangeReceiverQualifier", type: "text", required: true, maxLength: 2 },
		{ name: "InterchangeReceiverID", type: "text", required: true, maxLength: 15 },
		{ name: "RequestInterchangeAcknowledgement", type: "boolean" },
		{ name: "ApplicationSenderID", type: "text", maxLength: 15 },
		{ name: "ApplicationReceiverID", type: "text", maxLength: 15 },

		// Patient Statement
		{ name: "PayToName", type: "text", maxLength: 80 },
		{ name: "PayToAddressStreet1", type: "text" },
		{ name: "PayToAddressStreet2", type: "text" },
		{ name: "PayToAddressCity", type: "text" },
		{ name: "PayToAddressState", type: "text" },
		{ name: "PayToAddressZip", type: "text", maxLength: 10 },
		{ name: "PaymentPhoneNumber", type: "text", maxLength: 25 },
		{ name: "PaymentWebsite", type: "text", maxLength: 80 },
		{ name: "PaymentDueDate", type: "text" },
		{ name: "PatientStatementProviderDetailLocation", type: "tableItem", required: true, tableName: "FillInGroup~20613" },
		{ name: "PatientStatementLogoPath", type: "text", maxLength: 100 },

		// 270 options
		{ name: "CustomInformationSourceName", type: "text", maxLength: 60 },
		{ name: "InformationSourceIdQualifier", type: "text", maxLength: 2 },
		{ name: "CustomInformationSourceId", type: "text", maxLength: 80 },
		{ name: "InformationReceiverIdQualifier", type: "text", maxLength: 2 },
		{ name: "CustomInformationReceiverID", type: "text", maxLength: 80 },
		{ name: "IncludeTaxonomyCode", type: "boolean" },
		{ name: "CustomTaxonomyCode", type: "text", maxLength: 50 },
		{ name: "ServiceTypeCodes", type: "text" },

		// Claim options
		{ name: "ProviderDetailLocation", type: "tableItem", required: true, tableName: "FillInGroup~20613" },
		{ name: "TaxonomyCodeQualifier", type: "text", maxLength: 2 },
		{ name: "TaxonomyCode", type: "text", maxLength: 10 },
		{ name: "TypeOfCare", type: "tableItem", required: true, tableName: "FillInGroup~20636" },
		{ name: "StatementFromDate", type: "tableItem", required: true, tableName: "FillInGroup~20614" },
		{ name: "StatementThroughDate", type: "tableItem", required: true, tableName: "FillInGroup~20615" },
		{ name: "AllowMultiplePriorAuth", type: "boolean" },
		{ name: "TreatmentAuthorizationCode", type: "tableItem", required: true, tableName: "FillInGroup~20616" },
		{ name: "DiagnosesSource", type: "tableItem", required: true, tableName: "FillInGroup~20617" },
		{ name: "ProceduresSource", type: "tableItem", required: true, tableName: "FillInGroup~20617" },
		{ name: "TopServiceLineWithHipps", type: "boolean" },
		{ name: "TopServiceLineRevenueCode", type: "text", maxLength: 4, required: true },
		{ name: "TopServiceLineQuantity", type: "text", required: true },
		{ name: "TopServiceLineChargeAmount", type: "text", required: true },
		{ name: "DefaultHippsCode", type: "text", maxLength: 6 },
		{ name: "WeeklyReportingOfGIP", type: "boolean" },
		{ name: "RevenueCodeIncludeLeadingZero", type: "boolean" },
		{ name: "ServiceUnitCount", type: "tableItem", tableName: "FillInGroup~20618" },
		{ name: "ChargeAmount", type: "tableItem", tableName: "FillInGroup~20619" },

		// Scenarios
		{ name: "Scenario1", type: "boolean" }, // Include Occurrence Code 61 (or 62) w/ Inpatient Discharge Date when inpatient discharge has occurred within 14 days of bill period start
		{ name: "Scenario2", type: "boolean" }, // Include Occurrence Span Code 77 w/ Uncovered Date Range when NOE was submitted late
		{ name: "Scenario3", type: "boolean" }, // Include Occurrence Span Code M2 w/ Respite Care Date Range for each set Respite Care days when multiple sets of respite days are included on the claim
		{ name: "Scenario4", type: "boolean" }, // Include Occurrence Code 42 w/ Discharge Date when Discharge Reason has an HIS Reason for Discharge that is "02 - Revoked"
		{ name: "Scenario5", type: "boolean" }, // Include Condition Code 52 when Discharge Reason has an HIS Reason for Discharge that is "04 - Moved out of hospice service area"
		{ name: "Scenario6", type: "boolean" }, // Include Condition Code H2 when Discharge Reason has an HIS Reason for Discharge that is "06 - Discharge for cause"
		{ name: "Scenario7", type: "boolean" }, // Include Remark w/ Discharge Reason & Date when Discharge Reason has an HIS Reason for Discharge that is NOT "01 - Expired"
		{ name: "Scenario8", type: "boolean" }, // Include CBSA in value code(s) 61 and/or G8 based on Level of Care provided to patient
		{ name: "Scenario9", type: "boolean" }  // Include Occurrence Span Code 77 w/ Uncovered Date Range when RAP was submitted late
	]});

	var claimMappingTemplate = {
		noForm: false,
		containers: [{
			sections: [{
				hideDivider: false,
				rows: [{
					fields: [{
						name: "Insurances",
						displayName: "Insurances:",
						labelSize: "medium-large"
					}, {
						name: "copyButton",
						noField: true
					}]
				}]
			}, {
				hideDivider: false,
				rows: [{
					fields: [{
						name: "Name",
						displayName: "Name:",
						fieldSize: "largest",
						labelSize: "medium-large"
					}, {
						name: "Status",
						displayName: "Status:",
						labelSize: "smaller"
					}]
				}, {
					fields: [{
						name: "InsuranceType",
						displayName: "Insurance Type:",
						labelSize: "medium-large"
					}]
				}, {
					fields: [{
						name: "ClaimForm",
						displayName: "Claim Form:",
						fieldSize: "larger",
						labelSize: "medium-large"
					}, {
						name: "PrintWithFormImage",
						displayName: "Print w/ Form Image?",
						labelSize: "large",
						containerStyle: "display: none;"
					}]
				}]
			}]
		}, {
			attachPoint: "genericElectronicOptionsContainer",
			sections: [{
				//title: "Generic options",
				rows: [{
					orientation: "vertical",
					field: {
						name: "InterchangeSenderID",
						displayName: "Interchange Sender ID (Segment ISA, Element ISA06)",
						labelSize: "grander"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "InterchangeReceiverQualifier",
						displayName: "Interchange Receiver Qualifier (Segment ISA, Element ISA07)",
						labelSize: "grander"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "InterchangeReceiverID",
						displayName: "Interchange Receiver ID (Segment ISA, Element ISA08)",
						labelSize: "grander"
					}
				}, {
					field: {
						name: "RequestInterchangeAcknowledgement",
						displayName: "Request Interchange Acknowledgement? (Segment ISA, Element ISA14)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "ApplicationSenderID",
						displayName: "Application Sender ID (Segment GS, Element GS02)",
						labelSize: "grander"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "ApplicationReceiverID",
						displayName: "Application Receiver ID (Segment GS, Element GS03)",
						labelSize: "grander"
					}
				}]
			}]
		}, {
			attachPoint: "claimOptionsContainer",
			sections: [{
				title: "Provider Details  (837I: Locator 1 / Loop 2010AA) (837P: Locator 33 / Loop 2010AA)",
				rows: [{
					fields: [{
						name: "ProviderDetailLocation",
						displayName: "Provider Detail Location:",
						labelSize: "larger"
					}]
				}]
			}, {
				title: "Additional Provider Details (837I: Locator 81 / Loop 2000A, Segment PRV, Element PRV03) (837P: Locator 33b / Loop 2000A, Segment PRV, Element PRV03)",
				rows: [{
					fields: [{
						name: "TaxonomyCodeQualifier",
						displayName: "Taxonomy Code Qualifier:",
						labelSize: "larger",
						fieldSize: "smaller"
					}, {
						name: "TaxonomyCode",
						displayName: "Taxonomy Code:",
						labelSize: "large"
					}]
				}]
			}, {
				title: "Type of Bill (837I: Locator 4 / Loop 2300, Segment CLM, Element CLM05) (837P: Locator 24b / Loop 2300, Segment CLM, Element CLM05)",
				containerAttachPoint: "typeOfBillContainer",
				rows: [{
					fields: [{
						name: "TypeOfCare",
						displayName: "Type of Care:",
						labelSize: "medium",
						fieldSize: "medium"
					}]
				}]
			}, {
				title: "Statement Dates",
				rows: [{
					fields: [{
						name: "StatementFromDate",
						displayName: "Statement From Date:",
						labelSize: "large"
					}, {
						name: "StatementThroughDate",
						displayName: "Statement Through Date:",
						labelSize: "larger"
					}]
				}]
			}, {
				title: "Prior Authorizations (837I: Locator 63 / Loop 2300, Segment REF[G1], Element REF02) (837P: Locator 23 / Loop 2300, Segment REF[G1], Element REF02)",
				rows: [{
					fields: [{
						name: "AllowMultiplePriorAuth",
						displayName: "Claim may contain more than 1 prior authorization number",
						labelSize: "grander",
						fieldSize: "smallest"
					}]
				 }, {
					fields: [{
						name: "TreatmentAuthorizationCode",
						displayName: "Treatment Authorization Code:",
						labelSize: "larger"
					}]
				}]
			}, {
				title: "Diagnoses  (837I: Locator 66-67 / Loop 2300, Segment HI[ABK,ABF]) (837P: Locator 21 / Loop 2300, Segment HI[ABK,ABF])",
				rows: [{
					fields: [{
						name: "DiagnosesSource",
						displayName: "Diagnoses Source:",
						labelSize: "large"
					}]
				}]
			}, {
				title: "Procedures (837I: Locator 74 / Loop 2300, Segment HI[BBR,BBQ])",
				rows: [{
					fields: [{
						name: "ProceduresSource",
						displayName: "Procedures Source:",
						labelSize: "large"
					}]
				}]
			}, {
				title: "Service Lines (837I: Locator 42-49 / Loop 2400, Segment SV2) (837P: Locator 24 / Loop 2400, Segment SV1)",
				rows: [{
					fields: [{
						name: "TopServiceLineWithHipps",
						displayName: "Add top service line w/ HIPPS",
						labelSize: "largest",
						fieldSize: "smallest"
					}]
				}, {
					containerAttachPoint: "topServiceLineContainer",
					fields: [{
						name: "TopServiceLineRevenueCode",
						displayName: "Revenue Code:",
						labelSize: "medium-large",
						fieldSize: "small"
					}, {
						name: "TopServiceLineQuantity",
						displayName: "Quantity:",
						labelSize: "medium-small",
						fieldSize: "small"
					}, {
						name: "TopServiceLineChargeAmount",
						displayName: "Charge Amount:",
						labelSize: "medium-large",
						fieldSize: "small"
					}]
				}, {
					fields: [{
						name: "WeeklyReportingOfGIP",
						displayName: "Visit Reporting: Weekly reporting of GIP while in Q5006",
						labelSize: "grander",
						fieldSize: "smallest"
					}]
				}, {
					fields: [{
						name: "RevenueCodeIncludeLeadingZero",
						displayName: "Revenue Code Include Leading Zero",
						labelSize: "largest",
						fieldSize: "smallest"
					}]
				}, {
					fields: [{
						name: "ServiceUnitCount",
						displayName: "Service Unit Count:",
						labelSize: "large"
					}]
				}, {
					containerAttachPoint: "serviceUnitCountGridContainer"
				}, {
					fields: [{
						name: "ChargeAmount",
						displayName: "Charge Amount:",
						labelSize: "large"
					}]
				}, {
					fields: [{
						name: "DefaultHippsCode",
						displayName: "Default HIPPS Code (PDGM):",
						labelSize: "large",
						fieldSize: "small"
					}]
				}]
			}]
		}, {
			attachPoint: "eligibilityContainer",
			sections: [{
				//title: "",
				rows: [{
					orientation: "vertical",
					field: {
						name: "CustomInformationSourceName",
						displayName: "Custom Information Source Name (Loop 2100A, Segment NM1, Element NM103)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "InformationSourceIdQualifier",
						displayName: "Information Source ID Qualifier (Loop 2100A, Segment NM1, Element NM108)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "CustomInformationSourceId",
						displayName: "Custom Information Source ID (Loop 2100A, Segment NM1, Element NM109)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "InformationReceiverIdQualifier",
						displayName: "Information Receiver ID Qualifier (Loop 2100B, Segment NM1, Element NM108)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "CustomInformationReceiverID",
						displayName: "Custom Information Receiver ID (Loop 2100B, Segment NM1, Element NM109)",
						labelSize: "grandest"
					}
				}, {
					field: {
						name: "IncludeTaxonomyCode",
						displayName: "Include Taxonomy Code? (Loop 2100B, Segment PRV)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "CustomTaxonomyCode",
						displayName: "Custom Taxonomy Code (Loop 2100B, Segment PRV, Element PRV03)",
						labelSize: "grandest"
					}
				}, {
					orientation: "vertical",
					field: {
						name: "ServiceTypeCodes",
						displayName: "Service Type Codes (Loop 2110C, Segment EQ, Element EQ01)",
						labelSize: "grandest"
					}
				}]
			}]
		}, {
			attachPoint: "patientStatementContainer",
			sections: [{
				title: "Pay To",
				rows: [{
					fields: [{
						name: "PayToName",
						displayName: "Name",
						fieldSize: "grandest",
						labelSize: "smaller"
					}]
				}, {
					fields: [{
						name: "PayToAddressStreet1",
						displayName: "Street",
						labelSize: "smaller"
					}]
				}, {
					fields: [{
						name: "PayToAddressStreet2",
						displayName: "",
						labelSize: "smaller"
					}]
				}, {
					fields: [{
						name: "PayToAddressCity",
						displayName: "City",
						labelSize: "smaller"
					}]
				}, {
					fields: [{
						name: "PayToAddressState",
						displayName: "State",
						fieldSize: "small",
						labelSize: "smaller"
					}, {
						name: "PayToAddressZip",
						displayName: "Zip",
						fieldSize: "small",
						labelSize: "smallest"
					}]
				}, {
					fields: [{
						name: "PaymentPhoneNumber",
						displayName: "Payment Phone Number",
						labelSize: "medium-large"
					}]
				}, {
					fields: [{
						name: "PaymentWebsite",
						displayName: "Payment Website",
						fieldSize: "grandest",
						labelSize: "medium-large"
					}]
				}, {
					fields: [{
						name: "PaymentDueDate",
						displayName: "Payment Due",
						labelSize: "medium-large"
					}]
				}]
			}, {
				title: "Provider Details",
				rows: [{
					fields: [{
						name: "PatientStatementProviderDetailLocation",
						displayName: "Provider Detail Location:",
						labelSize: "larger"
					}]
				}, {
					fields: [{
						name: "PatientStatementLogoPath",
						displayName: "Logo Path:"
					}]
				}]
			}]
		}, {
			attachPoint: "additionalCodingContainer",
			sections: [{
				title: "Additional Coding",
				hideDivider: false,
				rows: [{
					fields: [{
						name: "Scenario1",
						displayName: "Include Occurrence Code 61 (or 62) w/ Inpatient Discharge Date when inpatient discharge has occurred within 14 days of bill period start",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario8",
						displayName: "Include CBSA in value code(s) 61 and/or G8 based on Level of Care provided to patient",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario9",
						displayName: "Include Occurrence Span Code 77 w/ Uncovered Date Range when RAP was submitted late",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario2",
						displayName: "Include Occurrence Span Code 77 w/ Uncovered Date Range when NOE was submitted late",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario3",
						displayName: "Include Occurrence Span Code M2 w/ Respite Care Date Range for each set Respite Care days when multiple sets of respite days are included on the claim",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario4",
						displayName: "Include Occurrence Code 42 w/ Discharge Date when Discharge Reason has an HIS Reason for Discharge that is \"02 - Revoked\"",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario5",
						displayName: "Include Condition Code 52 when Discharge Reason has an HIS Reason for Discharge that is \"04 - Moved out of hospice service area\"",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario6",
						displayName: "Include Condition Code H2 when Discharge Reason has an HIS Reason for Discharge that is \"06 - Discharge for cause\"",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					fields: [{
						name: "Scenario7",
						displayName: "Include Remark w/ Discharge Reason & Date when Discharge Reason has an HIS Reason for Discharge that is NOT \"01 - Expired\"",
						fieldSize: "smallest",
						labelSize: "grandest"
					}]
				}, {
					containerAttachPoint: "additionalCodingGridContainer"
				}]
			}]
		}]
	};

	ClaimMapping = claimMappingModel.getObject();
	var ClaimMappingEditor = declare([ObjectEditor], {
		objectModel: claimMappingModel,
		templateString: template.generate(claimMappingTemplate),
		requirementCollections: [],
		documentReqCollections: [],

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);

			t.getPropertyModel("PaymentDueDate").getField = function(node) {
				return new NumberTextBox(this.getFieldProperties(), node);
			};
			t.getPropertyModel("PaymentDueDate").getInternal = function(value) {
				return (value ? value : (value === 0 ? value : ""));
			};

			t.getPropertyModel("TopServiceLineQuantity").getField = function(node) {
				return new NumberTextBox(lang.mixin(this.getFieldProperties(), {constraints: {places: "0", min: 0, max: 99}}), node);
			};
			t.getPropertyModel("TopServiceLineQuantity").getInternal = function(value) {
				return (value ? value : (value === 0 ? value : ""));
			};

			t.getPropertyModel("TopServiceLineChargeAmount").getField = function(node) {
				return new NumberTextBox(lang.mixin(this.getFieldProperties(), {constraints: {places: "0,2"}}), node);
			};
			t.getPropertyModel("TopServiceLineChargeAmount").getInternal = function(value) {
				return (value ? value : (value === 0 ? value : ""));
			};

			// Override default behavior to allow carets in this field
			t.getPropertyModel("ServiceTypeCodes").getInternal = function(value) {
				var t = this;
				return (t._inLoad ? value : value.toString().replace(/(~|`)/g,""));
			};

			t.getPropertyModel("InsuranceType").noDefault = true;
		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);

			t.Insurances = new MultiSelect({disabled: true, style: "min-width: 300px;"}, t.InsurancesFieldContainer);

			t.copyClaimMappingButton = new Button({
				label: "Copy Claim Mapping",
				onClick: function() {
					var saveData = t._getSaveData();
					t.pageEditor.object.set("editMode", "add");
					delete saveData.Name;
					delete saveData.id;
					t.set("object", new ClaimMapping(saveData));
				}
			},t.copyButtonLabelContainer);

			domStyle.set(t.InterchangeSenderIDField.domNode, "width", "140px");
			domStyle.set(t.InterchangeReceiverQualifierField.domNode, "width", "50px");
			domStyle.set(t.InterchangeReceiverIDField.domNode, "width", "140px");
			domStyle.set(t.ApplicationSenderIDField.domNode, "width", "140px");
			domStyle.set(t.ApplicationReceiverIDField.domNode, "width", "140px");
			domStyle.set(t.TaxonomyCodeQualifierField.domNode, "width", "50px");
			domStyle.set(t.TaxonomyCodeField.domNode, "width", "110px");
			domStyle.set(t.PayToAddressStateField.domNode, "width", "50px");
			domStyle.set(t.PayToAddressZipField.domNode, "width", "100px");
			//domStyle.set(t.PayToNameField.domNode, "width", "250px");
			//domStyle.set(t.PaymentWebsiteField.domNode, "width", "250px"); 
			domStyle.set(t.PaymentDueDateField.domNode, "width", "50px");
			domStyle.set(t.TopServiceLineRevenueCodeField.domNode, "width", "50px");
			domStyle.set(t.TopServiceLineQuantityField.domNode, "width", "60px");
			domStyle.set(t.TopServiceLineChargeAmountField.domNode, "width", "60px");
			domStyle.set(t.topServiceLineContainer,"margin-left","40px");
			domStyle.set(t.DefaultHippsCodeField.domNode, "width", "60px");
			
			// Move checkboxes before labels
			arrayUtils.forEach(t.objectModel.getPropertyModels(),function(propertyModel) {
				if (propertyModel.type === "boolean") {
					var propName = propertyModel.name;
					
					var fieldContainer = t[propName + "FieldContainer"];
					var labelContainer = t[propName + "LabelContainer"];
					
					domStyle.set(fieldContainer, "min-width", "20px");
					domStyle.set(labelContainer, "width", "725px");

					domConstruct.place(fieldContainer, labelContainer, "before");
				}
			});

			domConstruct.create("label", { innerHTML: "&nbsp;days from Aging Date" }, t.PaymentDueDateFieldContainer);
			
			t.serviceUnitCountGrid = new ServiceUnitCountGrid();
			t.serviceUnitCountGrid.startup();
			domConstruct.place(t.serviceUnitCountGrid.domNode, t.serviceUnitCountGridContainer, "last");

			t.additionalCodingGrid = new AdditionalCodeGrid();
			t.additionalCodingGrid.startup();
			domConstruct.place(t.additionalCodingGrid.domNode, t.additionalCodingGridContainer, "last");
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.TaxonomyCodeQualifierField.set({ pattern: "[A-Za-z0-9]{2}" });
			t.InterchangeReceiverQualifierField.set({ pattern: "[A-Za-z0-9]{2}" });
			t.TaxonomyCodeField.set({ pattern: "[A-Za-z0-9]{10}" });
			t.PaymentDueDateField.set("constraints", { fractional: false, min: 0 });
			t.PayToAddressStateField.set("pattern", "[A-Z]{2}");
			t.PayToAddressStateField.set("uppercase", true);
			t.PayToAddressZipField.set("pattern", "\\d{5}(-\\d{4})?");
			t.PaymentPhoneNumberField.set("pattern", "(\\d{3}-)?\\d{3}-\\d{4}");
			t.PaymentPhoneNumberField.set("invalidMessage", "123-4567 or 123-456-7890 format");
			t.TopServiceLineRevenueCodeField.set({ regExp: revenueCodeRegEx, invalidMessage: revenueCodeInvalidMessage });
			t.ServiceTypeCodesField.set({pattern: "([A-Za-z0-9]{1,2})(\\^([A-Za-z0-9]{1,2})){0,98}",
										 invalidMessage: 'Enter the desired Service Type Code(s), separated by "^"' });
			t.DefaultHippsCodeField.set("uppercase", true);
			t.DefaultHippsCodeField.set({ pattern: "[A-Za-z0-9]{5,6}" });

			t.own(aspect.before(t, "_setObjectAttr", function(object) {
				if (!object.ProviderDetailLocation) {
					// default to Location
					object.ProviderDetailLocation = 3;
				}
				if (!object.AllowMultiplePriorAuth) {
					// default to true
					object.AllowMultiplePriorAuth = true;
				}
				if (!object.TreatmentAuthorizationCode) {
					// default to Authorization #
					object.TreatmentAuthorizationCode = 3;
				}
				if (!object.DiagnosesSource) {
					// default to POC / POF
					object.DiagnosesSource = 1;
				}
				if (!object.ProceduresSource) {
					// default to POC / POF
					object.ProceduresSource = 1; 
				}
				if (!object.RevenueCodeIncludeLeadingZero) {
					// default to true
					object.RevenueCodeIncludeLeadingZero = true;
				}
				if (!object.ServiceUnitCount) {
					// default to Per Charge
					object.ServiceUnitCount = 1; 
				}
				if (!object.ChargeAmount) {
					// default to Gross Rate
					object.ChargeAmount = 1; 
				}

				if (!object.InsuranceType) {
					object.InsuranceType = 1; // default to Homecare
				}
			}));

			t.own(aspect.after(t, "_setObjectAttr", function(object) {
				t.additionalCodingGrid.set("object", object.AdditionalCodes);
				t.additionalCodingGrid.refreshGridBody();

				t.serviceUnitCountGrid.set("object", object.ServiceUnitCounts);
				t.serviceUnitCountGrid.refreshGridBody();

				// Populate Insurances with those that use documentation requirements with this claim mapping
				query("option", t.Insurances.containerNode).forEach(function(option) {domConstruct.destroy(option);});
				TableManager.get("MappingByInsurance").query({
					mapping: object.id
				}).forEach(function(mapping) {
					domConstruct.create("option", {
						label:escapeText(mapping.name), id:mapping.id
					}, t.Insurances.containerNode);
				});
			}, true));

			t.registerChangeLogic("InsuranceType", function(value) {
				if (value != 2) {
					t.object.set("WeeklyReportingOfGIP", false);
				} else if (!t.inObjectLoad) {
					// Default to Yes
					t.object.set("WeeklyReportingOfGIP", true);
				}

				var displayHospice = (value == 2) ? "" : "none";
				var displayHomecare = (value == 1) ? "" : "none";
				domStyle.set(t.WeeklyReportingOfGIPContainer, "display", displayHospice);
				domStyle.set(t.Scenario1Container, "display", displayHomecare);
				domStyle.set(t.Scenario2Container, "display", displayHospice);
				domStyle.set(t.Scenario3Container, "display", displayHospice);
				domStyle.set(t.Scenario4Container, "display", displayHospice);
				domStyle.set(t.Scenario5Container, "display", displayHospice);
				domStyle.set(t.Scenario6Container, "display", displayHospice);
				domStyle.set(t.Scenario7Container, "display", displayHospice);
				domStyle.set(t.Scenario8Container, "display", displayHospice);
				domStyle.set(t.Scenario9Container, "display", displayHomecare);
				domStyle.set(t.typeOfBillContainer, "display", displayHospice);
				domStyle.set(t.TypeOfCareContainer, "display", displayHospice);

				t.require("TypeOfCare", (domStyle.get(t.claimOptionsContainer, "display") == "") ? false : true);

				if (value == 1) {
					t.object.set("Scenario2", false);
					t.object.set("Scenario3", false);
					t.object.set("Scenario4", false);
					t.object.set("Scenario5", false);
					t.object.set("Scenario6", false);
					t.object.set("Scenario7", false);
					t.object.set("Scenario8", false);
					t.object.set("TypeOfCare", "");
				} else if (value == 2) {
					t.object.set("Scenario1", false);
					t.object.set("Scenario9", false);
				}

				// Filter claim form by insurance type. No insurance type on the form means it's unrestricted
				var restrictIdArray = [];
				TableManager.get("Tables.Insurance.ClaimForm").query().forEach(function(claimForm) {
					var insuranceType = claimForm.InsuranceType;
					if (insuranceType == value || insuranceType == "" || value == "") {
						restrictIdArray.push(claimForm.id);
					}
				});

				// Remove current bill period length selection if invalid
				var newClaimFormVal = t.object.get("ClaimForm");
				if (arrayUtils.indexOf(restrictIdArray, newClaimFormVal) == -1) {
					newClaimFormVal = "";
				}
				t.populateSelect(t.ClaimFormField, TableManager.get("Tables.Insurance.ClaimForm"),
								 newClaimFormVal, true, restrictIdArray);
				t.object.set("ClaimForm", newClaimFormVal);

				t.showTopServiceLineFields();
			});

			t.registerChangeLogic("ClaimForm", function(value) {
				var claimForm = {};
				if (value) {
					claimForm = TableManager.get("Tables.Insurance.ClaimForm").get(value);
				}
				
				if (!claimForm.AllowFormImage) {
					t.object.set("PrintWithFormImage", "");
				}
				
				domStyle.set(t.PrintWithFormImageContainer, "display", claimForm.AllowFormImage ? "" : "none");

				var showClaimFormOptions = (claimForm.Type == "UB-04/CMS-1450" || claimForm.Type == "CMS-1500" ||
											claimForm.Type == "837I" || claimForm.Type == "837P");

				domStyle.set(t.claimOptionsContainer, "display", showClaimFormOptions ? "" : "none");
				domStyle.set(t.genericElectronicOptionsContainer, "display", claimForm.Electronic ? "" : "none");
				domStyle.set(t.additionalCodingContainer, "display", (claimForm.Electronic && (claimForm.Type != "270")) ? "" : "none");
				domStyle.set(t.patientStatementContainer, "display", (claimForm.Type == "patientStatement") ? "" : "none");
				domStyle.set(t.eligibilityContainer, "display", (claimForm.Type == "270") ? "" : "none");
				domStyle.set(t.DefaultHippsCodeContainer, "display", (claimForm.name == "Electronic RAP Claim (837I)" ? "" : "none"));

				t.require("ProviderDetailLocation", showClaimFormOptions);
				t.require("TypeOfCare", (showClaimFormOptions && (t.object.get("InsuranceType") == 2)));
				t.require("StatementFromDate", showClaimFormOptions);
				t.require("StatementThroughDate", showClaimFormOptions);
				t.require("TreatmentAuthorizationCode", showClaimFormOptions);
				t.require("DiagnosesSource", showClaimFormOptions);
				t.require("ProceduresSource", showClaimFormOptions);

				t.require("InterchangeSenderID", claimForm.Electronic);
				t.require("InterchangeReceiverQualifier", claimForm.Electronic);
				t.require("InterchangeReceiverID", claimForm.Electronic);

				t.require("PatientStatementProviderDetailLocation", (claimForm.Type == "patientStatement"));
				
				if (!showClaimFormOptions) {
					t.resetClaimOptions();
				}
				if (!claimForm.Electronic) {
					t.resetGeneralElectronicOptions();
					t.resetAdditionalCodingOptions();
				}
				if (claimForm.Type != "patientStatement") {
					t.resetPatientStatementOptions();
				}
				if (claimForm.Type != "270") {
					t.resetEligibilityOptions();
				}
				if (claimForm.name != "Electronic RAP Claim (837I)") {
					t.object.set("DefaultHippsCode", "");
				}

				t.showTopServiceLineFields();
			});

			t.registerChangeLogic("TopServiceLineWithHipps", function(value) {
				domStyle.set(t.topServiceLineContainer, "display", value ? "" : "none");
				t.require("TopServiceLineRevenueCode", value);
				t.require("TopServiceLineQuantity", value);
				t.require("TopServiceLineChargeAmount", value);
				if (t.object) {
					t.object.set("TopServiceLineRevenueCode","");
					t.object.set("TopServiceLineQuantity","");
					t.object.set("TopServiceLineChargeAmount","");
				}
			});

			t.registerChangeLogic("ServiceUnitCount", function(value) {
				domStyle.set(t.serviceUnitCountGridContainer, "display", value == 5 ? "" : "none");
				if (value != 5) {
					t.serviceUnitCountGrid.set("object", []);
				}
			});
		},

		showTopServiceLineFields: function() {
			var t = this;
			var show = t.object && t.object.InsuranceType == 1 && arrayUtils.indexOf([1,2,5,6],t.object.ClaimForm) >= 0;
			domStyle.set(t.TopServiceLineWithHippsContainer, "display", (show ? "" : "none"));
			if (!show) {
				t.object.set("TopServiceLineWithHipps", false);
			}
		},

		_setActionTypeAttr: function(value) {
			/*
				value = "Edit", "Add", or "View"
			*/
			
			var t = this;
			t.inherited(arguments);

			t.copyClaimMappingButton.set("disabled", (value != "Edit"));
			t.additionalCodingGrid.set("actionType", value);
			t.serviceUnitCountGrid.set("actionType", value);
		},

		okToSave: function() {
			var t = this;
			var ok = false;

			var nameIsUnique = true;
			if (t.actionType == "Add") {
				TableManager.get("Tables.Insurance.ClaimMapping").query().forEach(function(set) {
					if (set.name == t.object.get("Name")) {
						nameIsUnique = false;
					}
				});
			}

			if (!t.formNode.validate()) {
				showMessage("Please correct any invalid fields.");
			} else if (query(".pinkBackground").length > 0) {
				showMessage("Please fill in all required fields.");
			} else if (t.actionType === "Edit" && (!t.get("modified") && !t.additionalCodingGrid.get("modified") && !t.serviceUnitCountGrid.get("modified"))) {
				showMessage("You have not made any changes.");
			} else if (!nameIsUnique) {
				showMessage('"' + t.object.get("Name") + '" is already in use. Please choose a different name.');
			} else {
				ok = true;
			}
			return ok;
		},

		save: function() {
			var t = this;
			var deferred = new Deferred();
			var promise = deferred.promise;

			if (t.okToSave()) {
				var saveData = t._getSaveData();
				var options = {
					saveData: saveData
				};

				promise = serverRequest("Tables.Insurance.ClaimMapping", "Save", options, false, 15000).then(function() {
				}, function(error) {
					showMessage(error);
					throw error;
				});
			} else {
				deferred.reject();
			}
			return promise;
		},

		_getSaveData: function() {
			var t = this;
			var saveData = t.object.getData();
			saveData.AdditionalCodes = t.additionalCodingGrid.grid.store.query();
			saveData.ServiceUnitCounts = t.serviceUnitCountGrid.grid.store.query();
			return saveData;
		},
		
		resetClaimOptions: function() {
			var t = this;
			t.object.set("ProviderDetailLocation", 3);
			t.object.set("TaxonomyCodeQualifier", "");
			t.object.set("TaxonomyCode", "");
			t.object.set("TypeOfCare", "");
			t.object.set("StatementFromDate", "");
			t.object.set("StatementThroughDate", "");
			t.object.set("AllowMultiplePriorAuth", true);
			t.object.set("TreatmentAuthorizationCode", 3);
			t.object.set("DiagnosesSource", 1);
			t.object.set("ProceduresSource", 1);
			t.object.set("TopServiceLineWithHipps", false);
			t.object.set("TopServiceLineRevenueCode", "");
			t.object.set("TopServiceLineQuantity", "");
			t.object.set("TopServiceLineChargeAmount", "");
			t.object.set("WeeklyReportingOfGIP", true);
			t.object.set("RevenueCodeIncludeLeadingZero", true);
			t.object.set("ServiceUnitCount", 1);
			t.object.set("ChargeAmount", 1);

			t.serviceUnitCountGrid.set("object", []);
		},
		
		resetGeneralElectronicOptions: function() {
			var t = this;
			t.object.set("InterchangeSenderID", "");
			t.object.set("InterchangeReceiverQualifier", "");
			t.object.set("InterchangeReceiverID", "");
			t.object.set("RequestInterchangeAcknowledgement", false);
			t.object.set("ApplicationSenderID", "");
			t.object.set("ApplicationReceiverID", "");
		},
		
		resetAdditionalCodingOptions: function() {
			var t = this;
			t.object.set("Scenario1", false);
			t.object.set("Scenario2", false);
			t.object.set("Scenario3", false);
			t.object.set("Scenario4", false);
			t.object.set("Scenario5", false);
			t.object.set("Scenario6", false);
			t.object.set("Scenario7", false);
			t.object.set("Scenario8", false);

			t.additionalCodingGrid.set("object", []);
		},
		
		resetPatientStatementOptions: function() {
			var t = this;
			t.object.set("PayToName", "");
			t.object.set("PayToAddressStreet1", "");
			t.object.set("PayToAddressStreet2", "");
			t.object.set("PayToAddressCity", "");
			t.object.set("PayToAddressState", "");
			t.object.set("PayToAddressZip", "");
			t.object.set("PaymentPhoneNumber", "");
			t.object.set("PaymentWebsite", "");
			t.object.set("PaymentDueDate", "");
			t.object.set("PatientStatementProviderDetailLocation", 3);
			t.object.set("PatientStatementLogoPath", "");
		},
		
		resetEligibilityOptions: function() {
			var t = this;
			t.object.set("CustomInformationSourceName", "");
			t.object.set("InformationSourceIdQualifier", "");
			t.object.set("CustomInformationSourceId", "");
			t.object.set("InformationReceiverIdQualifier", "");
			t.object.set("CustomInformationReceiverID", "");
			t.object.set("IncludeTaxonomyCode", false);
			t.object.set("CustomTaxonomyCode", "");
			t.object.set("ServiceTypeCodes", "");
		}
	});

	var additionalCodeModel = new ObjectModel({ propertyDefinitions: [
		{ name: "id", type: "text" },
		{ name: "CodeType", type: "tableItem", tableName: "FillInGroup~20620", displayName: "Type", required: true },
		{ name: "ClaimType", type: "tableItem", tableName: "FillInGroup~20621" },
		{ name: "CodeQualifier", type: "text", displayName: "Code", required: true },
		{ name: "DateFromType", type: "tableItem", tableName: "FillInGroup~20622", displayName: "Date", required: true },
		{ name: "DateFrom", type: "date", required: true },
		{ name: "DateToType", type: "tableItem", tableName: "FillInGroup~20622", displayName: "End", required: true },
		{ name: "DateTo", type: "date", required: true },
		{ name: "ValueType", type: "tableItem", tableName: "FillInGroup~20623", displayName: "Value", required: true },
		{ name: "Value", type: "text", required: true }
	]});

	var additionalCodeTemplate = {
		noForm: false,
		containers: [{
			sections: [{
				hideDivider: true,
				rows: [{
					orientation: "vertical",
					fields: [{
						name: "CodeType",
						displayName: "Code Type"
					}, {
						name: "CodeQualifier",
						displayName: "Code/Qualifier"
					}]
				}, {
					orientation: "vertical",
					fields: [{
						name: "ClaimType",
						displayName: "Claim Type"
					}, {
						name: "DateFromType",
						displayName: "Date/From"
					}, {
						name: "DateFrom",
						hideLabel: true
					}]
				}, {
					orientation: "vertical",
					fields: [{
						name: "spacer1",
						noField: true,
						hideLabel: true
					}, {
						name: "DateToType",
						displayName: "Thru"
					}, {
						name: "DateTo",
						hideLabel: true
					}]
				}, {
					orientation: "vertical",
					fields: [{
						name: "spacer2",
						noField: true,
						hideLabel: true
					}, {
						name: "ValueType",
						displayName: "Value"
					}, {
						name: "Value",
						hideLabel: true
					}]
				}]
			}]
		}]
	};

	var AdditionalCode = additionalCodeModel.getObject();
	var AdditionalCodeEditor = declare([ObjectEditor], {
		objectModel: additionalCodeModel,
		templateString: template.generate(additionalCodeTemplate),
		collection: {},

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);

			t.getPropertyModel("DateFrom").fieldProperties.constraints = {min: MINDATE};
			t.getPropertyModel("DateTo").fieldProperties.constraints = {min: MINDATE};
		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);

			domStyle.set(t.DateFromField.domNode, "width", "105px");
			domStyle.set(t.DateToField.domNode, "width", "105px");
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.own(aspect.before(t, "_setObjectAttr", function(object) {

			}));

			t.own(aspect.after(t, "_setObjectAttr", function(object) {
				
			}, true));

			t.registerChangeLogic("CodeType", function(value) {
				t.disable("CodeQualifier", true);
				t.disable("DateFromType", true);
				t.disable("DateToType", true);
				t.disable("ValueType", true);

				if (!t.inObjectLoad) {
					t.object.set("CodeQualifier", "");
					t.object.set("DateFromType", "");
					t.object.set("DateToType", "");
					t.object.set("ValueType", "");
				}
				
				if (value == 1) { // Occurrence
					t.disable("CodeQualifier", false);
					t.disable("DateFromType", false);
				} else if (value == 2) { // Occurrence Span
					t.disable("CodeQualifier", false);
					t.disable("DateFromType", false);
					t.disable("DateToType", false);
				} else if (value == 3) { // Value
					t.disable("CodeQualifier", false);
					t.disable("ValueType", false);
				} else if (value == 4) { // Condition
					t.disable("CodeQualifier", false);
				} else if (value == 5) { // Code
					t.disable("CodeQualifier", false);
					t.disable("ValueType", false);
				}
			});

			t.registerChangeLogic("ClaimType", function(value) {

			});

			t.registerChangeLogic("CodeQualifier", function(value) {

			});

			t.registerChangeLogic("DateFromType", function(value) {
				if (value != 11) { // Custom
					t.object.set("DateFrom", "");
				}
				t.disable("DateFrom", (value != 11));
				t.require("DateFrom", (value == 11));
			});

			t.registerChangeLogic("DateFrom", function(value) {
				var constraints = t.DateToField.get("constraints");
				constraints.min = value;
				t.DateToField.set("constraints", constraints);
			});

			t.registerChangeLogic("DateToType", function(value) {
				if (value != 11) { // Custom
					t.object.set("DateTo", "");
				}
				t.disable("DateTo", (value != 11));
				t.require("DateTo", (value == 11));
			});

			t.registerChangeLogic("ValueType", function(value) {
				if (value != 6) { // Custom
					t.object.set("Value", "");
				}
				t.disable("Value", (value != 6));
				t.require("Value", (value == 6));
			});
		},

		okToSave: function() {
			var t = this;
			var ok = false;

			if (!t.formNode.validate() ) {
				showMessage("Please correct any invalid fields.");
			} else if (query(".pinkBackground", t.domNode).length > 0) {
				showMessage("Please fill in all required fields.");
			} else if (t.actionType === "Edit" && !t.get("modified")) {
				showMessage("You have not made any changes.");
			} else {
				ok = true;
			}
			return ok;
		}
	});

	var ItemGrid = declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin], {
		title: "Edit Item",
		gridColumns: null,
		ModelType: null,
		EditorType: null,
		ObjectType:null,
		
		grid: {},
		store: new Memory(),
		buttons: [],
		modified: false,

		templateString: '<div>'+
							'<div class="dcs-subsmall" data-dojo-attach-point="titleNode"></div>'+
							'<div data-dojo-attach-point="actions"></div>'+
							'<div data-dojo-attach-point="gridContainer"></div>'+
						'</div>',

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);
		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);

			t.createGrid();
			t.actionButtons();

			domStyle.set(t.grid.domNode, "width", "750px");
			domStyle.set(t.grid.domNode, "height", "150px");

			t.gridContainer.appendChild(t.grid.domNode);
			t.grid.startup();
		},

		postCreate: function() {
			var t = this;
			t.inherited("postCreate", arguments);

			t.grid.connect(t.grid, "onCellDblClick", lang.hitch(t, t.actionEdit));

			t.own(aspect.after(t, "_setObjectAttr", function(object) {
				t.grid.setStore(new Memory({ data: object }));
				t.refreshGridBody();
			}, true));

			t.refreshGridBody();
			t.grid.resize();
		},

		getLayout: function() {
			var t = this;
			var properties = t.gridColumns;
			return t.ModelType.getGridLayout(properties);
		},
		
		createGrid: function() {
			var t = this;

			t.grid = new Grid({
				cacheClass: Cache,
				store: t.store,
				structure: t.getLayout(),
				selectRowMultiple: false,
				selectRowTriggerOnCell: true,
				modules: [
					VScroller,
					SelectRow,
					RowHeader,
					IndirectSelect,
					SingleSort
				]
			});

			t.own(aspect.after(t.grid, "_setDisabledAttr", lang.hitch(t, function(disable) {
				if (disable) {
					for (var i = 0; i < t.grid.store.data.length; i++) {
						t.grid.select.row.unselectable[t.grid.store.data[i].id] = 1;
					}
				} else {
					t.grid.select.row.unselectable = {};
				}
				t.grid.body.refresh();
			}), true));
		},

		refreshGridBody: function() {
			var t = this;
			t.grid.select.row.clear();
			t.grid.vScroller.scrollToRow(0,1);
			t.grid.model.clearCache();
			t.grid.body.refresh();
		},

		_setActionTypeAttr: function(value) {
			/*
				value = "Edit", "Add", or "View"
			*/
			
			var t = this;
			t.inherited(arguments);
			t.grid.set("disabled",  (value == "View"));

			arrayUtils.forEach(t.buttons,function(button) {
				button.set("disabled", (value == "View"));
			});
		},

		openEditor: function(object, actionType) {
			var t = this;
			var editor = new t.EditorType();
			editor.set("object", object);
			editor.set("actionType", actionType);
			editor.set("collection", t.grid.store.data);
			//TODO editor.set("claimMappingEditor", t);

			var messageDialog = new Dialog({
				title: t.title,
				content: editor,
				style: "width: 755px;",
				onCancel: function() {
					this.destroyRecursive();
				}
			});

			var actionBar = domConstruct.toDom("<div class='dijitDialogPaneActionBar'></div>");
			domConstruct.place(actionBar,messageDialog.containerNode);
			
			var closeDialog = function() {
				messageDialog.hide();
				messageDialog.destroyRecursive();
			};

			new Button({
				label: "Save",
				onClick: function() {
					if (editor.okToSave()) {
						var data = editor.get("object").getData();
						if (typeof data.id == "undefined") {
							var maxId = 0;
							arrayUtils.forEach(t.grid.store.data, function(code) {
								maxId = Math.max(code.id, maxId);
							});
							data.id = maxId + 1;
						}
						t.grid.store.put(data);
						t.refreshGridBody();
						t.grid.select.row.selectById(data.id);
						closeDialog();
						t.modified = true;
					}
				}
			}).placeAt(actionBar);

			new Button({
				label: "Cancel",
				onClick: closeDialog
			}).placeAt(actionBar);
			messageDialog.show();
		},

		actionButtons: function() {
			var t = this;

			t.buttons.push(new Button({
				label: "Add",
				disabled: t.readOnly,
				onClick: function() { t.actionAdd(); }
			}).placeAt(t.actions));

			t.buttons.push(new Button({
				label: "Edit",
				disabled: t.readOnly,
				onClick: function() { t.actionEdit(); }
			}).placeAt(t.actions));

			t.buttons.push(new Button({
				disabled: t.readOnly,
				label: "Remove",
				onClick: function() { t.actionRemove(); }
			}).placeAt(t.actions));
		},

		actionAdd: function() {
			var t = this;
			t.openEditor(new t.ObjectType(), "Add");
		},

		actionEdit: function() {
			var t = this;
			var selection = t.grid.select.row.getSelected();
			if (selection.length > 0) {
				t.openEditor(new t.ObjectType(t.grid.store.get(selection)), "Edit");
			}
		},

		actionRemove: function() {
			var t = this;
			var selection = t.grid.select.row.getSelected();
			if (selection.length > 0) {
				t.grid.store.remove(selection);
				t.refreshGridBody();
				t.modified = true;
			}
		}
	});

	var AdditionalCodeGrid = declare(ItemGrid,{
		title: "Additional Code",
		gridColumns: ["CodeType", "CodeQualifier", "DateFromType", "DateToType", "ValueType"],
		ModelType: additionalCodeModel,
		EditorType: AdditionalCodeEditor,
		ObjectType: AdditionalCode,
		
		getLayout: function() {
			var t = this;
			var layout = t.inherited(arguments);

			var valueFormatter = function(object) {
				var valueName = this.id.substr(0, this.id.length - 4);
				var propertyModel = t.ModelType.getPropertyModel(this.id);
				var tableFormatter = lang.hitch(propertyModel, propertyModel.formatter);
				var valueFormatter = t.ModelType.getPropertyModel(valueName).formatter;
				var formattedValue;

				if (tableFormatter(object[this.id]) == "Custom") {
					formattedValue = valueFormatter(object[valueName]);
				} else {
					formattedValue = tableFormatter(object[this.id]);
				}
				
				return formattedValue;
			};

			layout[2].formatter = valueFormatter;
			layout[3].formatter = valueFormatter;
			layout[4].formatter = valueFormatter;

			return layout;
		}
	});
	
	var serviceUnitCountModel = new ObjectModel({ propertyDefinitions: [
		{ name: "id", type: "text" },
		{ name: "Default", type: "boolean", displayName: "Default" },
		{ name: "Discipline", type: "tableItem", tableName: "Tables.Discipline", displayName: "Discipline", required: true },
		{ name: "ServiceUnitCount", type: "tableItem", tableName: "FillInGroup~20618", displayName: "Service Unit Count", required: true }
	]});

	var serviceUnitCountTemplate = {
		noForm: false,
		containers: [{
			sections: [{
				hideDivider: true,
				rows: [{
					field: {
						name: "Default",
						displayName: "Default",
						fieldSize: "small",
						labelSize: "large"
					}
				}, { 
					field: {
						name: "Discipline",
						displayName: "Discipline",
						fieldSize: "large",
						labelSize: "large"
					}
				}, { 
					field: {
						name: "ServiceUnitCount",
						displayName: "Service Unit Count",
						fieldSize: "large",
						labelSize: "large"
					}
				}]
			}]
		}]
	};

	var ServiceUnitCount = serviceUnitCountModel.getObject();
	var ServiceUnitCountEditor = declare([ObjectEditor], {
		objectModel: serviceUnitCountModel,
		templateString: template.generate(serviceUnitCountTemplate),
		collection: {},

		postMixInProperties: function() {
			var t = this;
			t.inherited("postMixInProperties", arguments);

		},

		buildRendering: function() {
			var t = this;
			t.inherited("buildRendering", arguments);

		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.registerChangeLogic("Default", function(value) {
				t.disable("Discipline", (value == 1));
				if (!t.inObjectLoad) {
					t.object.set("Discipline","");
				}
			});
		},

		okToSave: function() {
			var t = this;
			var ok = false;

			if (!t.formNode.validate() ) {
				showMessage("Please correct any invalid fields.");
			} else if (query(".pinkBackground", t.domNode).length > 0) {
				showMessage("Please fill in all required fields.");
			} else if (t.actionType === "Edit" && !t.get("modified")) {
				showMessage("You have not made any changes.");
			} else {
				ok = true;
			}
			return ok;
		},

		_getObjectAttr: function() {
			var t = this;
			var object = t.inherited(arguments);
			if (typeof object.Default === "undefined") {
				object.Default = 0;
			}
			return object;
		},

		_setObjectAttr: function(object) {
			var t = this;
			t.inherited(arguments);

			// Exclude "Custom"
			var inclusions = [];
			var propertyModel = t.getPropertyModel("ServiceUnitCount");
			arrayUtils.forEach(propertyModel.getOptionStore().data,function(item) {
				if (item.name !== "Custom") {
					inclusions.push(item.id);
				}
			});
			t.populateSelect(t.ServiceUnitCountField,propertyModel.getOptionStore(),(object ? object.ServiceUnitCount : null),true,inclusions);
		},

		_setCollectionAttr: function(value) {
			var t = this;

			// Only allow a single default selection
			t.disable("Default",arrayUtils.some(value,function(item) { return item.Default; }));
			
			// Exclude disciplines that have already been added
			var inclusions = [];
			var propertyModel = t.getPropertyModel("Discipline");
			arrayUtils.forEach(propertyModel.getOptionStore().data,function(discipline) {
				if (!arrayUtils.some(value,function(item) { return (!t.object || t.object.id !== item.id) && item.Discipline == discipline.id; })) {
					inclusions.push(discipline.id);
				}
			});
			t.populateSelect(t.DisciplineField,propertyModel.getOptionStore(),(t.object ? t.object.Discipline : null),true,inclusions);
		}
	});
	
	var ServiceUnitCountGrid = declare(ItemGrid,{
		title: "Service Unit Count",
		gridColumns: ["Default","Discipline","ServiceUnitCount"],
		ModelType: serviceUnitCountModel,
		EditorType: ServiceUnitCountEditor,
		ObjectType: ServiceUnitCount
	});

	return ClaimMappingEditor;
});