/*
	Author: Ryan Witherell
	Created: 2020-12-30
	Purpose: JavaScript for the Statements button on AIO charting pages pages.
	History:
*/

require(['dojo/ready'], function(ready) {
	var MAX_LENGTH = 1000;
	
	ready(function() {
		// Requirements
		require([
		  "dojo/parser",
		  "dojo/dom",
		  "dijit/registry",
		  "dojo/_base/array",
		  "dojo/_base/lang",
		  "dojox/html/entities",
		  "dojo/query",
		  "dojo/on",
		  "dojo/dom-style",
		  "dijit/MenuItem",
		  "dijit/form/DropDownButton"
		], function (parser, dom, registry, array, lang, entities, query, on, domStyle, MenuItem) {
			var onUpdateDiscipline = function(dropDownButton) {
				var disciplineField = dom.byId(dropDownButton.disciplineFieldId);
				dropDownButton.set("disabled",!disciplineField.value);
				if (disciplineField.value) {
					array.forEach(dropDownButton.dropDown.getChildren(),function(option) {
						dropDownButton.dropDown.removeChild(option);
					});
					
					var count = 0;
					var items = statementList && statementList.children ? statementList.children : [];
					array.forEach(items, function(item) {
						if (item.discipline == disciplineField.value) {
							var newOption = new MenuItem({
								label: entities.encode(item.name),
								onClick: function(){addTextToInstructions(dropDownButton.textFieldId,item.text);}
							});
							dropDownButton.dropDown.addChild(newOption);
							count++;
						}
					});
					
					if (!count) {
						dropDownButton.dropDown.addChild(new MenuItem({label: "(None)", baseClass: "emphasize"}));
					}
				}
			}
			
			// Parse dojo
			parser.parse().then(function() {
				query(".additionalStatementsMenu").forEach(function(buttonNode) {
					var dropDownButton = registry.getEnclosingWidget(buttonNode);
					if (dropDownButton && dropDownButton.disciplineFieldId && dropDownButton.textFieldId) {
						on(dom.byId(dropDownButton.disciplineFieldId),"change",lang.partial(onUpdateDiscipline,dropDownButton));
						onUpdateDiscipline(dropDownButton);
					} else {
						domStyle.set(dropDownButton.domNode,"display","none");
					}
				});
			});

			function addTextToInstructions(fieldId,text) {
				var statementNode = dom.byId(fieldId);
				if (statementNode.disabled || statementNode.readOnly) {
					alert("The field is read only");
					return;
				}
				var existingText = statementNode.value;
				var textLength = (existingText+text).length;
				if (textLength > MAX_LENGTH) {
					alert("The instructions are too long to append this statement. Please shorten the instructions by "+(textLength - MAX_LENGTH)+" characters.");
				} else {
					if (existingText !== "") {
						text = " " + text;
					}
					statementNode.value = existingText+text;
					statementNode.onchange();
				}
			}
		});
	});
});