define([
    "dojo/_base/declare",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/select/Row",
    "gridx/modules/extendedSelect/Row",
    "gridx/modules/IndirectSelectColumn",
    "dojo/store/Memory",
    "thornberry/app/utils",
    "dojo/_base/array"
],function(
    declare,
    Grid,
    Cache,
    SingleSort,
    SelectRow,
    ExtendedSelectRow,
    IndirectSelectColumn,
    Memory,
    utils,
    arrayUtils
) {
    var gridRowClickHandler = function(event) {
        // Summary:
        //      When connected with the grid's onRowClick event, this causes clicking the checkbox to add the row to those already selected, and clicking anywhere else
        //      to result in only the clicked row being selected
        if (event.columnIndex !== 0 && this.select.row.enabled) {
            var selected = this.select.row.getSelected();
            if (selected.length === 1 && selected[0] === event.rowId) {
                this.select.row.deselectById(event.rowId);
            } else {
                if (!event.ctrlKey) {
                    this.select.row.clear();
                }
                this.select.row.selectById(event.rowId);
            }
        }
    };

    return declare([Grid],{
        selectableRows: false,
        groupCodeTable: null,

        cacheClass: null,
        store: null,
        structure: null,
        modules: null,
        style: "margin:8px",
        autoHeight: true,

        constructor: function() {
            this.cacheClass = Cache;
        },

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            if (!t.store) {
                t.store = new Memory();
            }

            t.modules = [SingleSort];
            if (t.selectableRows) {
                t.modules.push(SelectRow);
                t.modules.push(ExtendedSelectRow);
                t.modules.push(IndirectSelectColumn);
            }

            t.structure = [
                { id: "groupCodeDisplay", field: "groupCode", name: "Group Code", formatter: utils.formatters.tableItem("groupCode",t.groupCodeTable,"shortName") },
                { id: "reasonCode", field: "reasonCode", name: "CARC" },
                { id: "amount", field: "amount", name: "Amount", formatter: utils.formatters.dollars("amount") }
            ];
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            if (t.select) {
                t.connect(t,"onRowClick",gridRowClickHandler);

                t.connect(t,"onCellDblClick",function(evt) {
                    t.onRowDoubleClick(evt.rowId);
                });
            }
        },

        onRowDoubleClick: function(id) {

        },

        getSelected: function() {
            var t = this;
            var selectedIds = [];
            var rows = t.rows();
            for (var i = 0; i < rows.length; i++) {
                if (t.select.row.isSelected(rows[i].id)) {
                    selectedIds.push(rows[i].id);
                }
            }
            return selectedIds;
        },

        getAll: function() {
            return arrayUtils.map(this.rows(),function(row) {
                return row.id;
            });
        },
    });
});