define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_DataFormMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/ClaimEditor.html",
    "dojo/text!./Templates/PatientSection.html",
    "dojo/text!./Templates/PolicySection.html",
    "dojo/text!./Templates/ClaimSection.html",
    "dojo/text!./Templates/DiagnosisSection.html",
    "thornberry/object/TableManager",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/on",
    "dojo/aspect",
    "dojo/when",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/CellWidget",
    "dojo/store/Memory",
    "thornberry/app/Component/BillingManagement/CodeGrid",
    "thornberry/app/utils",
    "thornberry/app/Component/BillingManagement/PriorPayerEditor",
    "dijit/form/ValidationTextBox",
    "dijit/form/Textarea",
    "thornberry/app/Component/BillingManagement/CodeEditor"
],function(
    declare,
    lang,
    arrayUtils,
    _EditorComponent,
    _DataFormMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    patientSectionTemplate,
    policySectionTemplate,
    claimSectionTemplate,
    diagnosisSectionTemplate,
    TableManager,
    domStyle,
    domClass,
    domConstruct,
    on,
    aspect,
    when,
    Grid,
    Cache,
    CellWidget,
    Memory,
    CodeGrid,
    utils,
    PriorPayerEditor
) {
    return declare([_EditorComponent,_DataFormMixin,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: template,

        modifiers: null,
        dcns: null,

        selectedCodeIds: null,
        isEditingCode: false,
        codeStore: null,
        lastTempCodeId: 0,
        modifiedCodeStore: null,
        deletedCodeIds: null,
        priorPayerEditors: null,

        constructor: function() {
            var t = this;
            t.selectedCodeIds = [];
            t.codeStore = new Memory();
            t.modifiedCodeStore = new Memory();
            t.deletedCodeIds = [];
            t.priorPayerEditors = [];
        },

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.modifiers = {};
            t.dcns = {};

            t.templateString = lang.replace(template,{
                patientSectionTemplate: patientSectionTemplate,
                policySectionTemplate:  policySectionTemplate,
                claimSectionTemplate:   claimSectionTemplate,
                diagnosisSectionTemplate: diagnosisSectionTemplate
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var billRateStructure = [
                { id: "chargeCodeName", field: "chargeCodeName", name: "Charge Code" },
                { id: "effectiveFromDate", field: "effectiveFromDate", name: "Effective", formatter: utils.formatters.date("effectiveFromDate") },
                { id: "grossAmount", field: "grossAmount", name: "Gross", formatter: utils.formatters.dollars("grossAmount") },
                { id: "netRate", field: "netRate", name: "Net", formatter: utils.formatters.dollars("netRate") },  
                { id: "netRateTypeName", field: "netRateTypeName", name: "Rate Type" },
                { id: "roundingTypeName", field: "roundingTypeName", name: "Rounding" }    
            ];
            t.billRateGrid = utils.createGrid(billRateStructure,t.billRateGridContainer,true);

            var levelOfCareStructure = [
                { id: "startDate", field: "startDate", name: "Start Date", formatter: utils.formatters.date("startDate") },
                { id: "days", field: "days", name: "Days" },
                { id: "levelOfCare", field: "levelOfCare", name: "Level of Care" },
                { id: "hospiceLocation", field: "hospiceLocation", name: "Hospice Location" },
                { id: "hospiceFacility", field: "hospiceFacility", name: "Hospice Facility" },
                { id: "cbsa", field: "cbsa", name: "CBSA" },
                { id: "wageIndex", field: "wageIndex", name: "Wage Index" },
                { id: "qCode", field: "qCode", name: "Q Code" },
                { id: "npi", field: "npi", name: "NPI" }
            ];
            t.levelOfCareGrid = utils.createGrid(levelOfCareStructure,t.levelOfCareGridContainer);

            var modifiersDecorator = function() {
                return "<div><input data-dojo-type='dijit/form/ValidationTextBox' data-dojo-attach-point='modifiersField' data-dojo-props=\"pattern:'[a-zA-Z0-9]{2}(,[a-zA-Z0-9]{2}){0,3}',maxLength:11,uppercase:true,promptMessage:'Enter the modifiers required for inclusion with the HCPCS code on the claim.  Format: Modifier(s) must be a set of up to 4 modifiers, each 2 characters, separated by commas.',invalidMessage:'Enter the modifiers required for inclusion with the HCPCS code on the claim.  Format: Modifier(s) must be a set of up to 4 modifiers, each 2 characters, separated by commas.'\"><span data-dojo-attach-point='modifiersNode'></span></div>";
            };
            var modifiersSetCellValue = function(gridData,storeData,cellWidget) {
                var id = cellWidget.cell.row.id;
                var data = t.chargeGrid.store.get(id);
                var modifiersField = cellWidget.modifiersField;
                if (modifiersField._cnnt) {
                    modifiersField._cnnt.remove();
                }
                if (data.id === "summary"){
                    domStyle.set(modifiersField.domNode,"display","none");
                    domStyle.set(cellWidget.modifiersNode,"display","none");
                } else if (t.props.disabled) {
                    domStyle.set(modifiersField.domNode,"display","none");
                    domStyle.set(cellWidget.modifiersNode,"display","");
                   
                    var node = domConstruct.toDom("<span>"+(data.modifiers ? data.modifiers : "")+"</span>");
                    domConstruct.place(node,cellWidget.modifiersNode,"only");
                } else {
                    domStyle.set(modifiersField.domNode,"display","");
                    domStyle.set(cellWidget.modifiersNode,"display","none");
                    modifiersField._onChangeActive = false;
                    modifiersField.set("value",data.modifiers);
                    modifiersField._onChangeActive = true;

                    modifiersField._cnnt = on(modifiersField,"change",function() {
                        t.onModifiersChanged(id,modifiersField.get("value"));
                    });
                }
            };
            var chargeStructure = [
                { id: "type", field: "type", name: "Type" },
                { id: "description", field: "description", name: "Description" },
                { id: "chargeDate", field: "chargeDate", name: "Date", formatter: utils.formatters.date("chargeDate") },
                { id: "chargeCode", field: "chargeCode", name: "Charge Code" },  
                { id: "gross", field: "gross", name: "Gross", formatter: utils.formatters.dollars("gross"), style: "text-align: right" },
                { id: "net", field: "net", name: "Net", formatter: utils.formatters.dollars("net"), style: "text-align: right" },
                { id: "covered", field: "covered", name: "Covered?" },   
                { id: "durQty", field: "durQty", name: "Dur/Qty" },
                { id: "revenueCode", field: "revenueCode", name: "Revenue Code" },
                { id: "hcpcsCode", field: "hcpcsCode", name: "HCPCS Code" },
                { id: "modifiers", field: "modifiers", name: "Modifiers", widgetsInCell: true, decorator: modifiersDecorator, setCellValue: modifiersSetCellValue, width: "200px" }
            ];
            t.chargeGrid = new Grid({
                cacheClass: Cache,
                store: new Memory(),
                structure: chargeStructure,
                autoHeight: true,
                modules: [CellWidget]
            });
            t.chargeGrid.connect(t.chargeGrid.body, "onAfterRow", function(row) {
                var item = row.grid.model.byId(row.id).item;
                try {
                    if (item.id === "summary") {
                        domClass.add(row.node(),"bold");
                    }
                } catch (e) {console.error(e);}
            });
            t.chargeGrid.startup();
            t.chargeGridContainer.appendChild(t.chargeGrid.domNode);

            var dcnDecorator = function() {
                return "<div><input data-dojo-type='dijit/form/ValidationTextBox' data-dojo-attach-point='dcnField' data-dojo-props=\"maxLength:32\"><span data-dojo-attach-point='dcnNode'></span></div>";
            };
            var dcnSetCellValue = function(gridData,storeData,cellWidget) {
                var id = cellWidget.cell.row.id;
                var data = t.patientPoliciesGrid.store.get(id);
                var dcnField = cellWidget.dcnField;
                if (dcnField._cnnt) {
                    dcnField._cnnt.remove();
                }
                if (t.props.disabled) {
                    domStyle.set(dcnField.domNode,"display","none");
                    domStyle.set(cellWidget.dcnNode,"display","");
                   
                    var node = domConstruct.toDom("<span>"+(data.dcn ? data.dcn : "")+"</span>");
                    domConstruct.place(node,cellWidget.dcnNode,"only");
                } else {
                    domStyle.set(dcnField.domNode,"display","");
                    domStyle.set(cellWidget.dcnNode,"display","none");
                    dcnField._onChangeActive = false;
                    dcnField.set("value",data.dcn);
                    dcnField._onChangeActive = true;

                    dcnField._cnnt = on(dcnField,"change",function() {
                        t.onDcnChanged(id,dcnField.get("value"));
                    });
                }
            };
            var patientPoliciesStructure = [
                { id: "seq", field: "seq", name: "Seq" },
                { id: "insurance", field: "insurance", name: "Insurance" },
                { id: "policyNum", field: "policyNum", name: "Policy #" },
                { id: "effectiveDate", field: "effectiveDate", name: "Effective", formatter: utils.formatters.date("effectiveDate") },
                { id: "priorPayments", field: "priorPayments", name: "Prior Payments" },
                { id: "dcn", field: "dcn", name: "DCN", widgetsInCell: true, decorator: dcnDecorator, setCellValue: dcnSetCellValue, width: "200px" }
            ];
            t.patientPoliciesGrid = new Grid({
                cacheClass: Cache,
                store: new Memory(),
                structure: patientPoliciesStructure,
                autoHeight: true,
                modules: [CellWidget]
            });
            t.patientPoliciesGrid.startup();
            t.patientPoliciesGridContainer.appendChild(t.patientPoliciesGrid.domNode);
            //t.patientPoliciesGrid = utils.createGrid(patientPoliciesStructure,t.patientPoliciesGridContainer);

            when(TableManager.get("FillInGroup~20620"),function(table) {
                var grid = new CodeGrid({
                    typeTable: table,
                    store: t.codeStore,
                    selectableRows: true,
                    onRowDoubleClick: function(id) {
                        var code = t.codeStore.get(id);
                        if (!t.props.disabled && !t.isEditingCode && !code.isFromClaimMapping) {
                            t.editCode(id);
                        }
                    }
                });
                t.codeGridContainer.appendChild(grid.domNode);
                grid.startup();
                grid.connect(grid.select.row,"onSelectionChange",function() {t.onChangeSelection(grid.getSelected());});
                t.codeGrid = grid;
            });

            var submissionStructure = [
                { id: "dateTime", field: "dateTime", name: "Date/Time", formatter: function(itemData) { return itemData.date+" "+itemData.time; } },
                { id: "fileName", field: "fileName", name: "Filename" },
                { id: "form", field: "form", name: "Form" }
            ];
            t.submissionGrid = utils.createGrid(submissionStructure,t.submissionGridContainer);

            t.addRequirement(["typeOfBill"],true);
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);
            t.own(
                on(t.addCodeButton,"click",function() {t.editCode()}),
                on(t.editCodeButton,"click",function() {t.editCode(t.selectedCodeIds[0])}),
                on(t.deleteCodeButton,"click",function() {t.deleteCodes()}),
                aspect.after(t.codeStore,"setData",lang.hitch(t,"updateCodeGrid")),
                aspect.after(t.codeStore,"put",lang.hitch(t,"updateCodeGrid")),
                aspect.after(t.codeStore,"remove",lang.hitch(t,"updateCodeGrid"))
            );
        },

        refreshRendering: function() {
            var t = this;

            if (!t.value || !t.props.value || t.value.id != t.props.value.id) {
                // If the ID changes, then close the policy detail section
                domStyle.set(t.policyDetailSection,"display","none");
            }

            if (t.value !== t.props.value) {
                t.value = t.props.value;
                
                t.set("value",t.props.value);
                domStyle.set(t.billRateSection,"display",t.props.value && t.props.value.settings.patientNegotiatedBillRates == 1 ? "" : "none");
                domStyle.set(t.levelOfCareContainer,"display",t.props.value && t.props.value.settings.hospicePatient ? "" : "none");
                
                t.billRateGrid.setStore(new Memory({data:t.props.value ? t.props.value.insurance.billRates : []}));
                t.billRateGrid.model.clearCache();
                t.billRateGrid.body.refresh();

                t.levelOfCareGrid.setStore(new Memory({data:t.props.value ? t.props.value.levelsOfCare : []}));
                t.levelOfCareGrid.model.clearCache();
                t.levelOfCareGrid.body.refresh();

                t.chargeGrid.setStore(new Memory({data:t.props.value ? t.props.value.charges.charges : []}));
                t.chargeGrid.model.clearCache();
                t.chargeGrid.body.refresh();

                t.patientPoliciesGrid.setStore(new Memory({data:t.props.value ? t.props.value.policies : []}));
                t.patientPoliciesGrid.model.clearCache();
                t.patientPoliciesGrid.body.refresh();

                t.submissionGrid.setStore(new Memory({data:t.props.value ? t.props.value.submissions : []}));
                t.submissionGrid.model.clearCache();
                t.submissionGrid.body.refresh();

                domStyle.set(t.noSubmissionsMessage,"display",t.props.value && t.props.value.submissions.length ? "none" : "");
                domStyle.set(t.submissionGridContainer,"display",t.props.value && t.props.value.submissions.length ? "" : "none");

                domStyle.set(t.diagnosisContainer,"display",t.props.value && t.props.value.settings.patientStatement ? "none" : "");
                domStyle.set(t.codeContainer,"display",t.props.value && t.props.value.settings.patientStatement ? "none" : "");

                t.set("priorPayers",t.props.value ? t.props.value.priorPayers : []);
            } else {
                t.chargeGrid.model.clearCache();
                t.chargeGrid.body.refresh();

                t.patientPoliciesGrid.model.clearCache();
                t.patientPoliciesGrid.body.refresh();
            }

            var disabled = t.props.disabled;
            domStyle.set(t.typeOfBillNode,          "display",(disabled ? "" : "none"));
            domStyle.set(t.typeOfBillFieldContainer,"display",(disabled ? "none" : ""));

            domStyle.set(t.remarksNode,          "display",(disabled ? "" : "none"));
            domStyle.set(t.remarksFieldContainer,"display",(disabled ? "none" : ""));

            domStyle.set(t.replacedClaimContainer,"display",(t.props.value  && t.props.value.claim.replacedClaim != "" ? "" : "none"));
            domStyle.set(t.replacedByClaimContainer,"display",(t.props.value  && t.props.value.claim.replacedByClaims != "" ? "" : "none"));

            domStyle.set(t.noeStatusContainer,"display",(t.props.value  && t.props.value.claim.noeRequired == 1 ? "" : "none"));

            t.set("isEditingCode",false);

            arrayUtils.forEach(t.priorPayerEditors,function(editor) {
                editor.set("props", { disabled : disabled});
            });

            t.inherited(arguments);
        },

        updateModified: function(subClassValue) {
            var t = this;
            var modified = false;
            for (var id in t.modifiers) {
                modified = true;
                break;
            }
            for (var id in t.dcns) {
                modified = true;
                break;
            }
            modified = modified || (t.modifiedCodeStore.data.length || t.deletedCodeIds.length);
            modified = modified || (arrayUtils.some(t.priorPayerEditors,function(editor) { return editor.get("modified"); }));
            t.inherited(arguments,[modified]);
        },

        onModifiersChanged: function(claimId,value) {
            var t = this;
            var storeData = t.chargeGrid.store.get(claimId);
            if (storeData.modifiers == value) {
                delete t.modifiers[claimId];
            } else {
                t.modifiers[claimId] = value;
            }
            t.invalidate("modified");
        },

        onDcnChanged: function(policyId,value) {
            var t = this;
            var storeData = t.patientPoliciesGrid.store.get(policyId);
            if (storeData.dcn == value) {
                delete t.dcns[policyId];
            } else {
                t.dcns[policyId] = value;
            }
            t.invalidate("modified");
        },

        _setValueAttr: function(value) {
            var t = this;
            if (value != null) {
                value.typeOfBill = value.claim.typeOfBill;
                value.modifiedCodes = [];
                value.deletedCodeIds = [];

                var codes = (value && value.codes ? value.codes.slice() : []);
                t.codeStore.setData(codes);
                t.modifiedCodeStore.setData([]);
                t.deletedCodeIds = [];
            }
            t.inherited(arguments,[value]);
            t.modifiers = {};
            t.dcns = {};
        },

        _getValueAttr: function() {
            var t = this;
            var value = t.inherited(arguments);
            value.modifiedCodes = t.modifiedCodeStore.data;
            value.deletedCodeIds = t.deletedCodeIds.slice();
            return value;
        },

        _setPriorPayersAttr: function(value) {
            var t = this;
            while (t.priorPayerEditors.length > value.length) {
                t.priorPayerEditors.pop().destroyRecursive();
            }
            for (var i = 0; i < value.length; i++) {
                if (i < t.priorPayerEditors.length) {
                    t.priorPayerEditors[i].set("props",{ value: value[i], disabled: t.props.disabled });
                } else {
                    var editor = new PriorPayerEditor({ props: { value: value[i], disabled: t.props.disabled, onUpdateModified: function() { t.invalidate("modified");} } });
                    t.priorPayerContainer.appendChild(editor.domNode);
                    editor.startup();
                    t.priorPayerEditors.push(editor);
                }
            }
        },

        getSaveData: function() {
            var t = this;
            var value = t.get("value");
            value.id = t.props.value.id;
            value.modifiers = t.modifiers;
            value.dcns = t.dcns;
            value.priorPayers = [];
            arrayUtils.forEach(t.priorPayerEditors,function(editor) {
                if (editor.get("modified")) {
                    value.priorPayers.push(editor.get("value"));
                }
            });
            return value;
        },

        _togglePolicySection: function() {
            var t = this;
            var isHidden = domStyle.get(t.policyDetailSection,"display") === "none";
            domStyle.set(t.policyDetailSection,"display",isHidden ? "" : "none");
            t.resize();
        },

        updateCodeGrid: function() {
            var t = this;
            if (t.codeGrid) {
                t.codeGrid.select.row.clear();
                t.codeGrid.model.clearCache();
                t.codeGrid.body.refresh();
            }
        },

        updateCodeControls: function() {
            var t = this;
            domStyle.set(t.codeActionContainer,"display",t.props.disabled ? "none" : "");
            arrayUtils.forEach([t.addCodeButton,t.editCodeButton,t.deleteCodeButton],function(button) {
                button.set("disabled",t.props.disabled || t.isEditingCode);
            });

            var canEditCodes = t.selectedCodeIds.length && arrayUtils.every(t.selectedCodeIds,function(id) {
                return !t.codeStore.get(id).isFromClaimMapping;
            });


            domStyle.set(t.editCodeButton.domNode,"display",t.selectedCodeIds.length === 1 && canEditCodes ? "" : "none");
            domStyle.set(t.deleteCodeButton.domNode,"display",canEditCodes ? "" : "none");

            // Lock or unlock grid selections
            if (t.codeGrid) {
                if (t.isEditingCode || t.props.disabled) {
                    arrayUtils.forEach(t.codeStore.data,function(item) {
                        t.codeGrid.select.row.unselectable[item.id] = 1;
                    });
                } else {
                    t.codeGrid.select.row.unselectable = {};
                    t.codeEditor.set("props",{
                        disabled: true // disabled the editor so there will be no required fields
                    });
                }
                t.codeGrid.select.row.enabled = !t.isEditingCode && !t.props.disabled;
                t.codeGrid.body.refresh();
            }

            when(t.codeEditor.readyPromise,function() {
                // wait until the field state (disabled/required) has finished updating
                domStyle.set(t.codeEditorContainer,"display",(t.isEditingCode ? "" : "none"));
            });

            t.invalidate("fieldState");
        },

        onChangeSelection: function(selectedIds) {
            this.set("selectedCodeIds",selectedIds);
        },

        _setSelectedCodeIdsAttr: function(value) {
            var t = this;
            t.selectedCodeIds = value;
            t.updateCodeControls();
        },

        editCode: function(id) {
            var t = this;   
            if (!id) {
                t.codeGrid.select.row.clear();
            }
            
            t.codeEditor.set("props",{
                disabled: false,
                mode: (id ? "edit" : "add"),
                value: t.codeStore.get(id),
                onCancel: function() {
                    t.set("isEditingCode",false);
                },
                onApply: function() {
                    t.saveCode();
                }
            });
            t.codeHeader.innerText = (id ? "Edit" : "New")+" Additional Code";
            t.set("isEditingCode",true);
        },

        saveCode: function() {
            var t = this;
            if (!t.codeEditor.checkValid()) {
                utils.showMessage("Please correct any invalid fields.");
            } else if (!t.codeEditor.checkRequired()) {
                utils.showMessage("Please fill in all required fields.");
            } else {
                var data = lang.mixin(t.codeEditor.get("value"),{
                    id: (t.selectedCodeIds.length ? t.selectedCodeIds[0] : "T"+(++t.lastTempCodeId))
                });
                t.codeStore.put(data);
                domStyle.set(t.codeEditorContainer,"display","none"); // Hide immediately instead of waiting for updateCodeControls() to hide it so the screen updates uniformly
                t.modifiedCodeStore.put(data);
                t.set("isEditingCode",false);
                t.invalidate("modified");
                t.codeGrid.select.row.clear();
            }
        },

        deleteCodes: function() {
            var t = this;
            arrayUtils.forEach(t.selectedCodeIds,function(id) {
                try {t.codeStore.remove(id); } catch (e) {};
                t.modifiedCodeStore.remove(id);
                if (id.toString().charAt(0) !== "T") {
                    // Only set up for removal if not just added
                    t.deletedCodeIds.push(id);
                }
                t.codeGrid.select.row.clear();
            });
            t.invalidate("modified");
        },

        _setIsEditingCodeAttr: function(value) {
            var t = this;
            t.isEditingCode = value;
            t.updateCodeControls();
        },

        resize: function() {
            var t = this;
            t.billRateGrid.resize();
            t.levelOfCareGrid.resize();
            t.chargeGrid.resize();
            t.patientPoliciesGrid.resize();
            t.submissionGrid.resize();
            if (t.codeGrid) {
                t.codeGrid.resize();
            }
            arrayUtils.forEach(t.priorPayerEditors,function(editor) {
                editor.resize();
            });
        }
    });
});