define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_ViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/service/BillingManagement",
    "thornberry/app/service/Settings",
    "dojo/text!./Templates/PostToArView.html",
    "thornberry/app/utils",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/CellWidget",
    "dojo/store/Memory",
    "thornberry/report/formatter",
    "dojo/date",
    "dojo/dom-style",
    "dijit/form/Form",
    "dijit/form/CheckBox"
],function(
    declare,
    lang,
    arrayUtils,
    _EditorComponent,
    _ViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    service,
    settingsService,
    template,
    utils,
    Grid,
    Cache,
    SingleSort,
    CellWidget,
    Memory,
    formatter,
    date,
    domStyle
) {
    return declare([_EditorComponent,_ViewMixin,_TemplatedMixin,_WidgetsInTemplateMixin], {
        title: "Post to A/R",

        templateString: template,

        claimIds: null,
        completed: false,
        settings: null,

        settingsPromise: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                value: {
                    agingDate: utils.conversions.date("",new Date(),"get")
                },
                onComplete: null
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions: [
                    { label: "Cancel", onClick: t.props.onCloseView },
                    { label: "Post", onClick: function() {t.postToAr(); }},
                ],
                accountSummaryShown: false
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var decorator = function() {
                return "<input data-dojo-type='dijit/form/CheckBox' checked='true' data-dojo-attach-point='checkBox'>";
            };
            var setCellValue = function(gridData,storeData,cellWidget) {
                var column = cellWidget.cell.column.id;
                if (column === "include") {
                    cellWidget.checkBox.set("checked",true);
                } else if (column = "apply") {
                    var id = cellWidget.cell.row.id;
                    var data = t.grid.store.get(id);
                    
                    cellWidget.checkBox.set("checked",false);
                    domStyle.set(cellWidget.checkBox.domNode,"display",(!data.PrePayment ? "none" : ""));
                }
            };

            var structure = [
                { id: "patient", field: "PatientNameID", name: "Patient" },
                { id: "statementPeriod", field: "Statement Period", name: "Statement Period" },
                { id: "insurance", field: "Insurance", name: "Insurance" },
                { id: "type", field: "Type", name: "Type" },
                { id: "gross", field: "GrossRevenue", name: "Gross", formatter: formatter.getForColumn({field:"GrossRevenue"},{formatter:"dollars"}) },
                { id: "net", field: "NetRevenue", name: "Net", formatter: formatter.getForColumn({field:"NetRevenue"},{formatter:"dollars"}) },
                { id: "claimId", field: "Claim ID", name: "Claim ID" },
                { id: "include", field: "include", name: "Include in Claim File?", widgetsInCell: true, decorator: decorator, setCellValue: setCellValue, style: "text-align: center" },
                { id: "prePayment", field: "PrePayment", name: "Pre-Payment", formatter: formatter.getForColumn({field:"PrePayment"},{formatter:"dollars"}) },
                { id: "apply", field: "apply", name: "Apply", style: "text-align: center", widgetsInCell: true, decorator: decorator, setCellValue: setCellValue  }
            ];

            t.grid = new Grid({
                cacheClass: Cache,
                store: new Memory(),
                structure: structure,
                autoHeight: true,
                modules: [SingleSort,CellWidget],
                modelExtensions: ["gridx/core/model/extensions/FormatSort"]
            });
            t.claimGridContainer.appendChild(t.grid.domNode);
            t.grid.startup();

            t.addRequirement("agingDate",true);
        }, 

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                t.set("claimIds",t.props.claimIds);
            }
            if (!t.settings) {
                t.settingsPromise = settingsService.get("billingComponent").then(function(settings) {
                    t.settings = settings;
                });
                t.addReadyPromise(t.settingsPromise);
            }
            t.inherited(arguments);
        },

        _setClaimIdsAttr: function(value) {
            var t = this;            
            if (t.claimIds !== value) {
                t.settings = null; // Force this to refresh in case another financial organizational unit has been closed
                t.completed = false;
                t.grid.setStore(new Memory());
                t.grid.model.clearCache();
                t.grid.body.refresh();

                if (value) {
                    var promise = utils.fetchReportData("Billing.Report.UnbilledClaims",value,[
                        "PatientNameID",
                        "Statement Period",
                        "Insurance",
                        "Type",
                        "GrossRevenue",
                        "NetRevenue",
                        "Claim ID",
                        "PrePayment",
                        "OrganizationalUnit"
                    ]);
                    t.addReadyPromise(promise);
                    promise.then(function(results) {
                        t.grid.setStore(new Memory(results));
                        t.grid.model.clearCache();
                        t.grid.body.refresh();

                        t._setDateConstraint(results);
                    },utils.showMessage);
                }
                t.claimIds = value;
            }
        },

        _getValueAttr: function() {
            var t = this;
            var includedIds = [];
            var prepayIds = [];
            arrayUtils.forEach(t.grid.rows(),function(row) {
                if (row.grid.cellWidget.getCellWidget(row.id,"include").checkBox.get("checked")) {
                    includedIds.push(row.id);
                }
                if (row.grid.cellWidget.getCellWidget(row.id,"apply").checkBox.get("checked")) {
                    prepayIds.push(row.id);
                }
            });
            return lang.mixin(t.inherited(arguments),{
                claimIds: t.claimIds,
                includedIds: includedIds,
                prepayIds: prepayIds
            });
        },

        _setDateConstraint: function(claimStore) {
            var t = this;
            t.settingsPromise.then(function() {
                var minDate = null;
                var organizationalUnits = {};
                arrayUtils.forEach(claimStore.data,function(item) { 
                    if (!organizationalUnits[item.OrganizationalUnit]) {
                        var units = arrayUtils.filter(t.settings.financialOrganizationalUnits,function(financialOrganizationalUnit) {
                            return financialOrganizationalUnit.unit == item.OrganizationalUnit;
                        });
                        if (units.length && units[0].lastClosed) {
                            var closedDate = utils.conversions.date(null,units[0].lastClosed,"set");
                            if (minDate === null || closedDate.getTime() < minDate.getTime()) {
                                minDate = closedDate;
                            }
                        }
                        organizationalUnits[item.OrganizationalUnit] = true;
                    }
                });
                t.getField("agingDate").constraints.min = (minDate ? date.add(minDate,"day",1) : null);
            });
        },

        postToAr: function() {
            var t = this;
            if (!t.checkValid()) {
                utils.showMessage("Please correct any invalid fields.");
            } else if (!t.checkRequired()) {
                utils.showMessage("Please fill in all required fields.");
            } else {
                var value = t.get("value");

                // 1. Submit all selected claims
                var promise = service.submitClaims(value.claimIds,value.agingDate,value.prepayIds).then(function(errors) {
                    t.completed = true;
                    if (errors.length) {
                        var message = "";
                        arrayUtils.forEach(errors,function(error) {
                            message +="\n\n"+error.claimId+" - "+error.error;
                        });
                        utils.showMessage("There was a problem posting "+errors.length+" claim(s) to A/R. These will not be included in a claim file."+message,null,true,500);
                    }

                    // 2. Remove any claims that weren't submitted
                    var includedIds = arrayUtils.filter(value.includedIds,function(id) {
                        return arrayUtils.every(errors,function(error) { return error.id != id; });
                    });

                    if (includedIds.length > 0) {
                        // 3. Send all claims that were checked and didn't fail submission to Ensemble for claim file creation/submission. This is done by the PostingSummaryView
                        t.loadSubView("postingSummary",{ claimIds: includedIds, agingDate: value.agingDate });
                    } else {
                        t.props.onCloseView();
                        t.props.onComplete();
                    }
                },utils.showMessage);
                t.props.onSetLoadingPromise(promise);
            }
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);
            if (t.firstShow) {
                t.addSubView("postingSummary","thornberry/app/Component/BillingManagement/PostingSummaryView",function() { return {}; });
            }
            if (t.completed) {
                t.props.onCloseView();
            }
        },

        onBeforeClose: function() {
            var t = this;
            if (t.completed && t.props.onComplete) {
                t.props.onComplete();
                t.completed = false;
            }  
            return true;
        },

        resize: function() {
            var t = this;
            t.grid.resize();
        }
    })
});