define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/_base/lang",
    "dojo/text!./Templates/PriorPayerEditor.html",
    "thornberry/app/Component/BillingManagement/AdjustmentGrid",
    "thornberry/object/TableManager",
    "dojo/store/Memory",
    "dojo/when",
    "dojo/dom-style",
    "dojo/_base/array",
    "dojo/on",
    "dojo/aspect",
    "dijit/form/Form",
    "dijit/form/DateTextBox",
    "dijit/form/CurrencyTextBox",
    "thornberry/app/Component/BillingManagement/AdjustmentEditor"
],function(
    declare,
    _EditorComponent,
    _TableFieldEditorMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    lang,
    template,
    AdjustmentGrid,
    TableManager,
    Memory,
    when,
    domStyle,
    arrayUtils,
    on,
    aspect
) {
    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin], {
        templateString: template,

        selectedAdjustmentIds: null,
        isEditingAdjustment: false,
        adjustmentStore: null,
        lastTempAdjustmentId: 0,
        modifiedAdjustmentStore: null,
        deletedAdjustmentIds: null,
        priorPayerEditors: null,

        constructor: function() {
            var t = this;
            t.selectedAdjustmentIds = [];
            t.adjustmentStore = new Memory();
            t.modifiedAdjustmentStore = new Memory();
            t.deletedAdjustmentIds = [];
            t.priorPayerEditors = [];
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                value: null,
                disabled: true,
                onUpdateModified: null
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            when(TableManager.get("FillInGroup~20644"),function(table) {
                var grid = new AdjustmentGrid({
                    groupCodeTable: table,
                    store: t.adjustmentStore,
                    selectableRows: true,
                    onRowDoubleClick: function(id) {
                        if (!t.props.disabled && !t.isEditingAdjustment) {
                            t.editAdjustment(id);
                        }
                    }
                });
                t.gridContainer.appendChild(grid.domNode);
                grid.startup();
                grid.connect(grid.select.row,"onSelectionChange",function() {t.onChangeSelection(grid.getSelected());});
                t.grid = grid;

                t.updateControls();
            });

            t.addRequirement(null,true);
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);
            t.own(
                on(t.addButton,"click",function() {t.editAdjustment()}),
                on(t.editButton,"click",function() {t.editAdjustment(t.selectedAdjustmentIds[0])}),
                on(t.deleteButton,"click",function() {t.deleteAdjustments()}),
                aspect.after(t.adjustmentStore,"setData",lang.hitch(t,"updateGrid")),
                aspect.after(t.adjustmentStore,"put",lang.hitch(t,"updateGrid")),
                aspect.after(t.adjustmentStore,"remove",lang.hitch(t,"updateGrid"))
            );
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);

            if (t.props.value != t.value) {
                t.headerNode.innerText = (t.props.value ? t.props.value.orderDisplay+" Payer Payment/Adjustment Details - "+t.props.value.payerName+" ("+t.props.value.claimId+")" : "");
            }
        },

        _setValueAttr: function(value) {
            var t = this;
            if (value != null) {
                value.modifiedAdjustments = [];
                value.deletedAdjustmentIds = [];

                var adjustments = (value && value.adjustments ? value.adjustments.slice() : []);
                t.adjustmentStore.setData(adjustments);
                t.modifiedAdjustmentStore.setData([]);
                t.deletedAdjustmentIds = [];
            }
            t.inherited(arguments,[value]);
        },

        _getValueAttr: function() {
            var t = this;
            var value = t.inherited(arguments);
            value.id = t.props.value.id;
            value.policyId = t.props.value.policyId;
            value.adjustments = t.props.value.adjustments;
            value.modifiedAdjustments = t.modifiedAdjustmentStore.data;
            value.deletedAdjustmentIds = t.deletedAdjustmentIds.slice();
            return value;
        },

        updateModified: function(subClassValue) {
            var t = this;
            var modified = (t.modifiedAdjustmentStore.data.length || t.deletedAdjustmentIds.length);
            t.inherited(arguments,[modified]);
            if (t.props.onUpdateModified) {
                t.props.onUpdateModified();
            }
        },

        updateGrid: function() {
            var t = this;
            if (t.grid) {
                try { t.grid.select.row.clear(); } catch (e) {};
                t.grid.model.clearCache();
                t.grid.body.refresh();
            }
        },

        updateControls: function() {
            var t = this;
            domStyle.set(t.actionContainer,"display",t.props.disabled ? "none" : "");
            arrayUtils.forEach([t.addButton,t.editButton,t.deleteButton],function(button) {
                button.set("disabled",t.props.disabled || t.isEditingAdjustment);
            });

            domStyle.set(t.editButton.domNode,"display",t.selectedAdjustmentIds.length === 1 ? "" : "none");
            domStyle.set(t.deleteButton.domNode,"display",t.selectedAdjustmentIds.length > 0 ? "" : "none");

            // Lock or unlock grid selections
            if (t.grid) {
                if (t.isEditingAdjustment || t.props.disabled) {
                    arrayUtils.forEach(t.adjustmentStore.data,function(item) {
                        t.grid.select.row.unselectable[item.id] = 1;
                    });
                } else {
                    t.grid.select.row.unselectable = {};
                    t.editor.set("props",{
                        disabled: true // disabled the editor so there will be no required fields
                    });
                }
                t.grid.select.row.enabled = !t.isEditingAdjustment && !t.props.disabled;
                t.grid.body.refresh();
            }

            when(t.editor.readyPromise,function() {
                // wait until the field state (disabled/required) has finished updating
                domStyle.set(t.editorContainer,"display",(t.isEditingAdjustment ? "" : "none"));
            });

            t.invalidate("fieldState");
        },

        onChangeSelection: function(selectedIds) {
            this.set("selectedAdjustmentIds",selectedIds);
        },

        _setSelectedAdjustmentIdsAttr: function(value) {
            var t = this;
            t.selectedAdjustmentIds = value;
            t.updateControls();
        },

        editAdjustment: function(id) {
            var t = this;   

            if (!id) {
                try { t.grid.select.row.clear(); } catch (e) {};
            }
            
            t.editor.set("props",{
                disabled: false,
                mode: (id ? "edit" : "add"),
                value: t.adjustmentStore.get(id),
                onCancel: function() {
                    t.set("isEditingAdjustment",false);
                },
                onApply: function() {
                    t.saveAdjustment();
                }
            });
            t.header.innerText = (id ? "Edit" : "New")+" Claim Adjustment";
            t.set("isEditingAdjustment",true);
        },

        saveAdjustment: function() {
            var t = this;
            if (!t.editor.checkValid()) {
                utils.showMessage("Please correct any invalid fields.");
            } else if (!t.editor.checkRequired()) {
                utils.showMessage("Please fill in all required fields.");
            } else {
                var data = lang.mixin(t.editor.get("value"),{
                    id: (t.selectedAdjustmentIds.length ? t.selectedAdjustmentIds[0] : "T"+(++t.lastTempAdjustmentId))
                });
                t.adjustmentStore.put(data);
                domStyle.set(t.editorContainer,"display","none"); // Hide immediately instead of waiting for updateControls() to hide it so the screen updates uniformly
                t.modifiedAdjustmentStore.put(data);
                t.set("isEditingAdjustment",false);
                t.invalidate("modified");
                try { t.grid.select.row.clear(); } catch (e) {};
            }
        },

        deleteAdjustments: function() {
            var t = this;
            arrayUtils.forEach(t.selectedAdjustmentIds,function(id) {
                try {t.adjustmentStore.remove(id); } catch (e) {};
                t.modifiedAdjustmentStore.remove(id);
                if (id.toString().charAt(0) !== "T") {
                    // Only set up for removal if not just added
                    t.deletedAdjustmentIds.push(id);
                }
                try { t.grid.select.row.clear(); } catch (e) {};
            });
            t.invalidate("modified");
        },

        _setIsEditingAdjustmentAttr: function(value) {
            var t = this;
            t.isEditingAdjustment = value;
            t.updateControls();
        },

        resize: function() {
            var t = this;
            if (t.grid) {
                t.grid.resize();
            }
        },

        startup: function() {
            var t = this;
            t.inherited(arguments);
        }
    });
});