define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "thornberry/app/component/Report/View",
    "thornberry/app/utils",
    "thornberry/serverRequest"
],function(
    declare,
    lang,
    arrayUtils,
    ReportView,
    utils,
    serverRequest
) {
    return declare(ReportView,{
        title: "Void Queue",
        showPresetControl: false,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Billing.Report.VoidQueue"
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;

            var canDismiss = selectedIds.length && arrayUtils.every(selectedIds,function(id) {
                return t.getGridData(id).Dismissed === "No"; 
            });
            var canRestore = selectedIds.length && arrayUtils.every(selectedIds,function(id) {
                return t.getGridData(id).Dismissed === "Yes"; 
            });

            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh",             onClick: lang.hitch(t,"refreshData") },
                    [
                        { label: "Dismiss",     onClick: function() { t.dismiss(selectedIds,true); },    hidden: !canDismiss },
                        { label: "Restore",     onClick: function() { t.dismiss(selectedIds,false); },    hidden: !canRestore },
                        { label: "Void",        onClick: function() { t.loadSubView("void"); },    hidden: !selectedIds.length }
                    ]
                ]
            });
        },

        onRowDoubleClick: function(id) {
            
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);

            if (t.firstShow) {
                /*t.addSubView("addEditCharge","thornberry/app/Component/BillingVoidQueue/...",function() {
                    return {
                        mode: "edit", 
                        viewOnly: false,
                        onSave: lang.hitch(t,"refreshData")
                    };
                });*/
                t.addSubView("void","thornberry/app/Component/BillingManagement/VoidClaimView",function() {
                    return {
                        claimIds: t.props.selectedIds,
                        onComplete: lang.hitch(t,"refreshData")
                    };
                });
            }
        },

        refreshData: function() {
            var t = this;
            if (t.controller) {
                // Force the summarization grid to update
                t.controller.modified.filters = true;
            }
            t.inherited(arguments);
        },

        dismiss: function(ids,dismiss) {
            serverRequest("Billing.Tracking.Utility","DismissFromWeb",{ ids: ids, dismiss: dismiss }).then(lang.hitch(this,"refreshData"));
        }
    });
});