define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "dojo/_base/lang",
    "thornberry/app/Service/PatientInsurance",
    "thornberry/app/Component/PatientInsurance/AuthorizationEditor",
    "thornberry/app/utils",
    "dojo/Deferred"
],function(
    declare,
    _Component,
    _EditorViewMixin,
    lang,
    patientInsuranceService,
    AuthorizationEditor,
    utils,
    Deferred
) {
    return declare([_Component,_EditorViewMixin], {
        // properties
        title: "Authorization",
        helpPath: "/help/PatientInsurance.html#authorization-editor",
        
        lockCheckEnabled: true,
        objectType: "Patient.Insurance.Authorizations",

        EditorClass: AuthorizationEditor,
        editor: null,

        // internals
        ndocNum: "",
        authId: "",
        auth: null,
        policies: null,
        accountNum: "",

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                ndocNum: "",
                mode: "edit", // view or edit
                onSave: null
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                printButtonShown: !!t.props.objectId,
                report: {
                    reportClass: "Report.Definition.PatientInsuranceAuthWithHistory",
                    getFilters: lang.hitch(t,"getReportFilters")
                }
            });
        },

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                t.set("ndocNum",t.props.ndocNum);
                t.set("authId",t.props.objectId);

                t.editor.set("props",{
                    value: t.auth,
                    policies: t.policies,
                    mode: (t.props.objectId == "" ? "add" : "edit"),
                    onPrintHistory: lang.hitch(t,"printHistory"),
                    disabled: (t.props.mode === "view")
                });
            }
            t.inherited(arguments);
        },

        onShow: function() {
            this.editor.resize();
        },

        onWindowResize: function() {
            this.editor.resize();
        },

        onBeforeClose: function() {
            var t = this;
            if (t.editor.isEditingDetail && t.editor.authDetailEditor.get("modified")) {
                utils.showMessage("Please finish editing disciplines / supplies.");
                return false;
            }
            return t.inherited(arguments);
        },

        _setNdocNumAttr: function(value) {
            var t = this;
            if (value && t.ndocNum !== value) {
                t.addReadyPromise(patientInsuranceService.getPolicies(value));
                t.setHandle("watchPolicies",patientInsuranceService.watchPolicies(t.props.ndocNum,lang.hitch(t,"updatePolicies")));

                patientInsuranceService.getGeneral(value).then(function(result) {
                    t.accountNum = result.accountNum; // get the account num for report filter display
                });
            }
            t.ndocNum = value;
        },
        
        updatePolicies: function(data) {
            var t = this;
            t.policies = data;
            t.editor.set("props",{ policies: data });
        },

        _setAuthIdAttr: function(value) {
            var t = this;
            if (value && t.authId !== value) {
                t.authId = value;

                t.addReadyPromise(patientInsuranceService.getAuthorization(t.ndocNum,value).then(function(auth) {
                    if (t.authId === value) {
                        t.set("auth",auth);
                    }
                }));
            } else if (!value) {
                t.set("auth",null);
            }
            t.authId = value;
        },

        _setAuthAttr: function(value) {
            var t = this;
            t.auth = value;
            t.editor.set("props",{
                value: value
            });
        },

        startSave: function() {
            var t = this;
            if (t.editor.isEditingDetail && t.editor.authDetailEditor.get("modified")) {
                utils.showMessage("Please finish editing disciplines / supplies.");
                var deferred = new Deferred();
                deferred.reject();
                return deferred;
            } 
            return t.inherited(arguments);
        },

        save: function() {
            var t = this;
            var data = lang.mixin(t.editor.get("value"),{
                ndocNum: t.props.ndocNum,
                id: t.props.objectId
            });
            delete data.details; // Only need modifiedDetails and deletedDetailIds
            if (t.props.onSave) {
                t.props.onSave();
            }
            return patientInsuranceService.saveAuthorization(data).then(function() {
                patientInsuranceService.getAuthorizations(t.props.ndocNum,true);
                t.authId = ""; // Force the editor to reload the policy if the same policy is edited immediately after
            });
        },

        getReportFilters: function(presentation,report) {
            var t = this;
            var filters = [
                {
                    column: "NDocNumber",
                    displayName: "Patient",
                    value: [ t.props.ndocNum ],
                    displayValue: t.accountNum
                }
            ];
            if (report.className === "Report.Definition.PatientInsuranceAuthWithHistory") {
                var selectedIds = t.editor.authDetailGrid.getSelected(); // Get current value since t.props.selectedIds doesn't update on sort
                if (t.auth.allServices) {
                    selectedIds.push(t.auth.id+"||1");
                } else if (t.auth.details.length === 0) {
                    selectedIds.push(t.auth.id);
                }
                filters.push({
                    column: "InsuranceAuthId",
                    value: [t.props.objectId],
                    hidden: true
                });
                filters.push({
                    column: "id",
                    value: (selectedIds.length ? selectedIds : t.editor.authDetailGrid.getAll()),
                    hidden: true
                })
            };
            return filters;
        },

        printHistory: function() {
            var t = this;
            t.props.onGenerateReport({
                reportClass: "Report.Definition.PatientInsuranceAuthWithHistory",
                getFilters: lang.hitch(t,"getReportFilters"),
                getOptions: function() { return { HistoryHeader: true };}
            });
        }
    });
});